#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    # CM script internal variables
    env = i['env']

    extra_cache_tags = []
    if env.get('CM_EXTRA_CACHE_TAGS', '').strip() == '':
        print('')
        extra_cache_tags_str = input(
            'Enter extra tags for the custom CACHE entry separated by comma: ')

        extra_cache_tags = extra_cache_tags_str.strip().split(',')

    return {'return': 0, 'add_extra_cache_tags': extra_cache_tags}


def postprocess(i):

    env = i['env']

    path = env.get('CM_CUSTOM_CACHE_ENTRY_PATH', '').strip()

    if path != '':
        if not os.path.isdir(path):
            os.makedirs(path)
    else:
        path = os.getcwd()

    x = ''
    env_key = env.get('CM_CUSTOM_CACHE_ENTRY_ENV_KEY', '')
    if env_key != '':
        x = env_key + '_'

    env['CM_CUSTOM_CACHE_ENTRY_{}PATH'.format(x)] = path
    env['CM_CUSTOM_CACHE_ENTRY_PATH'] = path

    env_key2 = env.get('CM_CUSTOM_CACHE_ENTRY_ENV_KEY2', '')
    v = env.get(env_key2, '')
    real_path = v if v != '' else path

    env['CM_CUSTOM_CACHE_ENTRY_{}REAL_PATH'.format(x)] = real_path

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import cmind as cm


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    inp = i['input']
    if 'CM_FLASH_BOARD' in env:
        script_tags = "flash,tiny"
    else:
        script_tags = "reproduce,tiny,mlperf,octoml"
    boards = ["NUCLEO", "NRF"]
    microtvm_variants = {
        "cmsis_nn": [
            "ad", "ic", "vww", "kws"], "native": [
            "ic", "ad", "vww", "kws"]}
    for board in boards:
        for microtvm_variant in microtvm_variants:
            if board == "NRF" and microtvm_variant == "native":
                continue
            for model in microtvm_variants[microtvm_variant]:
                variation_tags_string = "_" + board + ",_" + microtvm_variant + ",_" + model
                tags = script_tags + "," + variation_tags_string
                if 'CM_RECREATE_BINARY' in env:
                    r = cm.access(
                        {'action': 'rm', 'automation': 'cache', 'tags': tags, 'force': 'true'})
                    if r['return'] > 0:
                        return r
                r = cm.access({'action': 'run', 'automation': 'script', 'tags': tags, 'quiet': 'true', 'env': env,
                               'input': inp, 'state': state, 'add_deps': inp.get('add_deps', {}), 'add_deps_recursive':
                               inp.get('add_deps_recursive', {})})
                if r['return'] > 0:
                    return r

    return {'return': 0}


def postprocess(i):
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    ckpt_path = env.get('CM_BERT_PRUNE_CKPT_PATH', '')
    if ckpt_path == '':
        p = env['CM_ML_MODEL_FILE_WITH_PATH']
        x = os.listdir(p)
        for y in x:
            if y.startswith('models--'):
                z = os.path.join(p, y)
                if os.path.isdir(z):
                    z1 = os.path.join(z, 'snapshots')
                    if os.path.isdir(z1):
                        z2 = os.listdir(z1)
                        if len(z2) > 0:
                            ckpt_path = os.path.join(z1, z2[0])

    env['CM_BERT_PRUNE_CKPT_PATH'] = ckpt_path

    out_dir = env.get('CM_BERT_PRUNE_OUTPUT_DIR', '')
    if out_dir == '':
        out_dir = os.path.join(os.getcwd(), 'pruned-model-output')
    env['CM_BERT_PRUNE_OUTPUT_DIR'] = out_dir

    print('')
    print(
        'Local CM cache path to the updated BERT pruner src from NeurIPS 2022: ' +
        env['CM_GIT_REPO_BERT_PRUNER_NEURIPS_2022_CHECKOUT_PATH'])

    print('')
    for k in ["CM_ML_MODEL_FILE_WITH_PATH",
              "CM_BERT_PRUNE_CKPT_PATH", "CM_BERT_PRUNE_OUTPUT_DIR"]:
        print('ENV["{}"]: {}'.format(k, env[k]))

    print('')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    print("Entered postprocess")

    return {'return': 0}



# Universal experiment automation to support universal benchmarking
# and optimization of apps and systems
#
# Author: Grigori Fursin
# Contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os
import itertools
import copy
import json

from cmind.automation import Automation
from cmind import utils


class CAutomation(Automation):
    """
    CM "experiment" automation actions
    """

    CM_RESULT_FILE = 'cm-result.json'
    CM_INPUT_FILE = 'cm-input.json'
    CM_OUTPUT_FILE = 'cm-output.json'

    ############################################################
    def __init__(self, cmind, automation_file):
        super().__init__(cmind, __file__)

    ############################################################
    def test(self, i):
        """
        Test automation

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          automation (str): automation as CM string object

          parsed_automation (list): prepared in CM CLI or CM access function
                                    [ (automation alias, automation UID) ] or
                                    [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]

          (artifact) (str): artifact as CM string object

          (parsed_artifact) (list): prepared in CM CLI or CM access function
                                    [ (artifact alias, artifact UID) ] or
                                    [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action
        """

        import json
        print(json.dumps(i, indent=2))

        return {'return': 0}

    ############################################################

    def run(self, i):
        """
        Run experiment

        Args:
          (CM input dict):

            (out) (str): if 'con', output to console

            (artifact) (str): experiment artifact name (can include repository separated by :)
            (tags) (str): experiment tags separated by comma

            (dir) (str): force recording into a specific directory


            (script) (str): find and run CM script by name
            (s)

            (script_tags) (str): find and run CM script by tags
            (stags)

            (rerun) (bool): if True, rerun experiment in a given entry/directory instead of creating a new one...

            (explore) (dict): exploration dictionary

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action
        """

        # Copy of original input
        ii_copy = copy.deepcopy(i)
        cur_dir = os.getcwd()

        # Find or add artifact based on repo/alias/tags
        r = self._find_or_add_artifact(i)
        if r['return'] > 0:
            return r

        experiment = r['experiment']

        console = i.get('out', '') == 'con'

        # Print experiment folder
        experiment_path = experiment.path

        if console:
            print('')
            print('Path to CM experiment artifact: {}'.format(experiment_path))

        # Get directory with datetime
        datetime = i.get('dir', '')

        if datetime == '' and i.get('rerun', False):
            # Check if already some dir exist

            directories = os.listdir(experiment_path)

            datetimes = sorted([f for f in directories if os.path.isfile(
                os.path.join(experiment_path, f, self.CM_RESULT_FILE))], reverse=True)

            if len(datetimes) == 1:
                datetime = datetimes[0]
            elif len(datetimes) > 1:
                print('')
                print('Select experiment:')

                datetimes = sorted(datetimes)

                num = 0
                print('')
                for d in datetimes:
                    print('{}) {}'.format(num, d.replace('.', ' ')))
                    num += 1

                if not console:
                    return {
                        'return': 1, 'error': 'more than 1 experiment found.\nPlease use "cm rerun experiment --dir={date and time}"'}

                print('')
                x = input('Make your selection or press Enter for 0: ')

                x = x.strip()
                if x == '':
                    x = '0'

                selection = int(x)

                if selection < 0 or selection >= num:
                    selection = 0

                datetime = datetimes[selection]

        if datetime != '':
            experiment_path2 = os.path.join(experiment_path, datetime)
        else:
            num = 0
            found = False

            while not found:
                r = utils.get_current_date_time({})
                if r['return'] > 0:
                    return r

                datetime = r['iso_datetime'].replace(
                    ':', '-').replace('T', '.')

                if num > 0:
                    datetime += '.' + str(num)

                experiment_path2 = os.path.join(experiment_path, datetime)

                if not os.path.isdir(experiment_path2):
                    found = True
                    break

                num += 1

        # Check/create directory with date_time
        if not os.path.isdir(experiment_path2):
            os.makedirs(experiment_path2)

        # Change current path
        print('Path to experiment: {}'.format(experiment_path2))

        os.chdir(experiment_path2)

        # Record experiment input with possible exploration
        experiment_input_file = os.path.join(
            experiment_path2, self.CM_INPUT_FILE)
        experiment_result_file = os.path.join(
            experiment_path2, self.CM_RESULT_FILE)

        # Clean original input
        for k in ['parsed_artifact', 'parsed_automation', 'cmd']:
            if k in ii_copy:
                del (ii_copy[k])

        r = utils.save_json(file_name=experiment_input_file, meta=ii_copy)
        if r['return'] > 0:
            return r

        # Prepare run command
        cmd = ''

        unparsed = i.get('unparsed_cmd', [])
        if len(unparsed) > 0:
            for u in unparsed:
                if ' ' in u:
                    u = '"' + u + '"'
                cmd += ' ' + u

            cmd = cmd.strip()

        # Prepare script run
        env = i.get('env', {})

        ii = {'action': 'native-run',
              'automation': 'script,5b4e0237da074764',
              'env': env}

        # Prepare exploration
        # Note that from Python 3.7, dictionaries are ordered so we can define order for exploration in json/yaml
        # ${{XYZ}} ${{ABC(range(1,2,3))}}

        # Extract exploration expressions from {{VAR{expression}}}
        explore = i.get('explore', {})

        j = 1
        k = 0
        while j >= 0:
            j = cmd.find('}}}', k)
            if j >= 0:
                k = j + 1

                l = cmd.rfind('{{', 0, j)

                if l >= 0:
                    l2 = cmd.find('{', l + 2, j)
                    if l2 >= 0:
                        k = l2 + 1

                        var = cmd[l + 2:l2]
                        expr = cmd[l2 + 1:j]

                        explore[var] = expr

                        cmd = cmd[:l2] + cmd[j + 1:]

        # Separate Design Space Exploration into var and range
        explore_keys = []
        explore_dimensions = []

        for k in explore:
            v = explore[k]

            explore_keys.append(k)


if not isinstance(v,             if )                v = eval(v)

            explore_dimensions.append(v)

        # Next command will run all iterations so we need to redo above command
        # once again
        step = 0

        steps = itertools.product(*explore_dimensions)

        num_steps = len(list(steps))

        steps = itertools.product(*explore_dimensions)

        ii_copy = copy.deepcopy(ii)

        for dimensions in steps:

            step += 1

            print('================================================================')
            print('Experiment step: {} out of {}'.format(step, num_steps))

            print('')

            ii = copy.deepcopy(ii_copy)

            env = ii.get('env', {})

            l_dimensions = len(dimensions)
            if l_dimensions > 0:
                print('  Updating ENV variables during exploration:')

                print('')
                for j in range(l_dimensions):
                    v = dimensions[j]
                    k = explore_keys[j]
                    print('    - Dimension {}: "{}" = {}'.format(j, k, v))

                    env[k] = str(v)

                print('')

            # Generate UID and prepare extra directory:
            r = utils.gen_uid()
            if r['return'] > 0:
                return r

            uid = r['uid']

            experiment_path3 = os.path.join(experiment_path2, uid)
            if not os.path.isdir(experiment_path3):
                os.makedirs(experiment_path3)

            # Get date time of experiment
            r = utils.get_current_date_time({})
            if r['return'] > 0:
                return r

            current_datetime = r['iso_datetime']

            # Change current path
            print('Path to experiment step: {}'.format(experiment_path3))
            print('')
            os.chdir(experiment_path3)

            # Prepare and run experiment in a given placeholder directory
            os.chdir(experiment_path3)

            ii['env'] = env

            # Change only in CMD
            env_local = {'CD': cur_dir,
                         'CM_EXPERIMENT_STEP': str(step),
                         'CM_EXPERIMENT_PATH': experiment_path,
                         'CM_EXPERIMENT_PATH2': experiment_path2,
                         'CM_EXPERIMENT_PATH3': experiment_path3}

            # Update {{}} in CMD
            cmd_step = cmd

            j = 1
            k = 0
            while j >= 0:
                j = cmd_step.find('{{', k)
                if j >= 0:
                    k = j
                    l = cmd_step.find('}}', j + 2)
                    if l >= 0:
                        var = cmd_step[j + 2:l]

                        # Such vars must be in env
                        if var not in env and var not in env_local:
                            return {
                                'return': 1, 'error': 'key "{}" is not in env during exploration'.format(var)}

                        if var in env:
                            value = env[var]
                        else:
                            value = env_local[var]

                        cmd_step = cmd_step[:j] + str(value) + cmd_step[l + 2:]

            ii['command'] = cmd_step

            print('Generated CMD:')
            print('')
            print(cmd_step)
            print('')

            # Prepare experiment step input
            experiment_step_input_file = os.path.join(
                experiment_path3, self.CM_INPUT_FILE)

            r = utils.save_json(file_name=experiment_step_input_file, meta=ii)
            if r['return'] > 0:
                return r

            experiment_step_output_file = os.path.join(
                experiment_path3, self.CM_OUTPUT_FILE)
            if os.path.isfile(experiment_step_output_file):
                os.delete(experiment_step_output_file)

            # Run CMD
            rr = self.cmind.access(ii)
            if rr['return'] > 0:
                return rr

            # Record output
            result = {}

            if os.path.isfile(experiment_step_output_file):
                r = utils.load_json(file_name=experiment_step_output_file)
                if r['return'] > 0:
                    return r

                result = r['meta']

                # Try to flatten
                try:
                    flatten_result = flatten_dict(result)
                    result = flatten_result
                except BaseException:
                    pass

            # Add extra info
            result['uid'] = uid
            result['iso_datetime'] = current_datetime

            # Attempt to append to the main file ...
            all_results = []

            if os.path.isfile(experiment_result_file):
                r = utils.load_json(file_name=experiment_result_file)
                if r['return'] > 0:
                    return r

                all_results = r['meta']

            all_results.append(result)

            r = utils.save_json(
    file_name=experiment_result_file,
     meta=all_results)
            if r['return'] > 0:
                return r

        rr = {'return': 0,
              'experiment_path': experiment_path,
              'experiment_path2': experiment_path2}

        return rr

    ############################################################

    def rerun(self, i):
        """
        Rerun experiment

        cm run experiment --rerun=True ...
        """

        i['rerun'] = True

        return self.run(i)

    ############################################################

    def replay(self, i):
        """
        Replay experiment

        Args:
          (CM input dict):

             (out) (str): if 'con', output to console

             (artifact) (str): experiment artifact

             (tags) (str): experiment tags separated by comma

             (dir) (str): experiment directory (often date time)
             (uid) (str): unique ID of an experiment

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action
        """

        # Find or add artifact based on repo/alias/tags
        i['fail_if_not_found'] = True
        r = self._find_or_add_artifact(i)
        if r['return'] > 0:
            return r

        experiment = r['experiment']

        console = i.get('out', '') == 'con'

        # Print experiment folder
        experiment_path = experiment.path

        if console:
            print('')
            print('Path to CM experiment artifact: {}'.format(experiment_path))

        # Check date and time folder
        uid = i.get('uid', '')
        datetime = i.get('dir', '')

        if datetime != '':
            datetimes = [datetime]
        else:
            directories = os.listdir(experiment_path)

            datetimes = sorted([f for f in directories if os.path.isfile(
                os.path.join(experiment_path, f, self.CM_RESULT_FILE))], reverse=True)

        if len(datetimes) == 0:
            return {'return': 1, 'error': 'experiment(s) not found in {}'.format(
                experiment_path)}

        # Check datetime directory
        found_result = {}

        if uid != '':
            for d in datetimes:
                r = self._find_uid({'path': experiment_path, 'datetime': d, 'uid': uid})
                if r['return'] > 0:
                    return r

                if len(r.get('result', {})) > 0:
                    found_result = r['result']
                    datetime = d
                    experiment_path2 = os.path.join(experiment_path, datetime)
                    break

            if len(found_result) == 0:
                return {'return': 1, 'error': 'couldn\'t find result with UID {} in {}'.format(
                    uid, experiment_path)}

        else:
            if len(datetimes) == 1:
                datetime = datetimes[0]
            else:
                print('')
                print('Available experiments:')

                datetimes = sorted(datetimes)

                num = 0
                print('')
                for d in datetimes:
                    print('{}) {}'.format(num, d.replace('.', ' ')))
                    num += 1

                if not console:
                    return {
                        'return': 1, 'error': 'more than 1 experiment found.\nPlease use "cm run experiment --dir={date and time}"'}

                print('')
                x = input('Make your selection or press Enter for 0: ')

                x = x.strip()
                if x == '':
                    x = '0'

                selection = int(x)

                if selection < 0 or selection >= num:
                    selection = 0

                datetime = datetimes[selection]

            # Final path to experiment
            experiment_path2 = os.path.join(experiment_path, datetime)

            if not os.path.isdir(experiment_path2):
                return {'return': 1, 'error': 'experiment path not found {}'.format(
                    experiment_path2)}

            r = self._find_uid({'path': experiment_path, 'datetime': datetime})
            if r['return'] > 0:
                return r

            results = r['meta']

            if len(results) == 0:
                return {'return': 1, 'error': 'results not found in {}'.format(
                    experiment_path2)}

            elif len(results) == 1:
                selection = 0

            else:
                print('')
                print('Available Unique IDs of results:')

                results = sorted(results, key=lambda x: x.get('uid', ''))

                num = 0
                print('')
                for r in results:
                    print('{}) {}'.format(num, r.get('uid', '')))
                    num += 1

                if not console:
                    return {
                        'return': 1, 'error': 'more than 1 result found.\nPlease use "cm run experiment --uid={result UID}"'}

                print('')
                x = input('Make your selection or press Enter for 0: ')

                x = x.strip()
                if x == '':
                    x = '0'

                selection = int(x)

                if selection < 0 or selection >= num:
                    selection = 0

            found_result = results[selection]
            uid = found_result['uid']

        # Final info
        if console:
            print('')
            print('Path to experiment: {}'.format(experiment_path2))

            print('')
            print('Result UID: {}'.format(uid))

        # Attempt to load cm-input.json
        experiment_input_file = os.path.join(
            experiment_path2, self.CM_INPUT_FILE)

        if not os.path.isfile(experiment_input_file):
            return {
                'return': 1, 'error': '{} not found - can\'t replay'.format(self.CM_INPUT_FILE)}

        r = utils.load_json(experiment_input_file)
        if r['return'] > 0:
            return r

        cm_input = r['meta']

        tags = cm_input.get('tags', '').strip()
        if 'replay' not in tags:
            if tags != '':
                tags += ','
            tags += 'replay'
        cm_input['tags'] = tags

        if console:
            print('')
            print('Experiment input:')
            print('')
            print(json.dumps(cm_input, indent=2))
            print('')

        # Run experiment again
        r = self.cmind.access(cm_input)
        if r['return'] > 0:
            return r

        # TBA - validate experiment, etc ...

        return {'return': 0}

    ############################################################

    def _find_or_add_artifact(self, i):
        """
        Find or add experiment artifact (reused in run and reply)

        Args:
          (CM input dict):

            (fail_if_not_found) (bool) - if True, fail if experiment is not found

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          experiment (CM artifact class): Experiment artifact

        """

        console = i.get('out', '') == 'con'

        # Try to find experiment artifact by alias and/or tags
        ii = utils.sub_input(i, self.cmind.cfg['artifact_keys'] + ['tags'])
        ii['action'] = 'find'

        ii_copy = copy.deepcopy(ii)

        # If artifact is specified, remove tags
        artifact = ii.get('artifact', '').strip()
        if artifact != '' and not artifact.endswith(':') \
                and '*' not in artifact and '?' not in artifact:
            if 'tags' in ii:
                del (ii['tags'])

        r = self.cmind.access(ii)
        if r['return'] > 0:
            return r

        lst = r['list']

        if len(lst) > 1:
            print('More than 1 experiment artifact found:')

            lst = sorted(lst, key=lambda x: x.path)

            num = 0
            print('')
            for e in lst:
                print('{}) {}'.format(num, e.path))
                print(
    '        Tags: {}'.format(
        ','.join(
            e.meta.get(
                'tags',
                 []))))
                num += 1

            if not console:
                return {'return': 1, 'error': 'more than 1 experiment artifact found.\nPlease use "cm run experiment {name}" or "cm run experiment --tags={tags separated by comma}"'}

            print('')
            x = input('Make your selection or press Enter for 0: ')

            x = x.strip()
            if x == '':
                x = '0'

            selection = int(x)

            if selection < 0 or selection >= num:
                selection = 0

            experiment = lst[selection]

        elif len(lst) == 1:
            experiment = lst[0]
        else:
            # Create new entry
            if i.get('fail_if_not_found', False):
                return {'return': 1, 'error': 'experiment not found'}

            ii = copy.deepcopy(ii_copy)
            ii['action'] = 'add'
            r = self.cmind.access(ii)
            if r['return'] > 0:
                return r

            experiment_uid = r['meta']['uid']

            r = self.cmind.access({'action': 'find',
                                   'automation': 'experiment,a0a2d123ef064bcb',
                                   'artifact': experiment_uid})
            if r['return'] > 0:
                return r

            lst = r['list']
            if len(lst) == 0 or len(lst) >1:
                return {
                    'return': 1, 'error': 'created experiment artifact with UID {} but can\'t find it - weird'.format(experiment_uid)}

            experiment = lst[0]

        return {'return': 0, 'experiment': experiment}

    ############################################################
    def _find_uid(self, i):
        """
        Find experiment result with a given UID

        Args:
          (CM input dict):

            path (str): path to experiment artifact
            datetime (str): sub-path to experiment
            (uid) (str): experiment UID

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          path_to_file (str): path to experiment result file
          meta (dict): complete list of all results
          result (dict): result dictionary with a given UID

        """

        path = i['path']
        datetime = i['datetime']
        uid = i.get('uid', '').strip()

        path_to_experiment_result_file = os.path.join(
            path, datetime, self.CM_RESULT_FILE)

        rr = {'return': 0, 'path_to_file': path_to_experiment_result_file}

        if os.path.isfile(path_to_experiment_result_file):
            r = utils.load_json(file_name=path_to_experiment_result_file)
            if r['return'] > 0:
                return r

            meta = r['meta']

            rr['meta'] = meta

            # Searching for UID
            if uid != '':
                for result in meta:
                    ruid = result.get('uid', '').strip()
                    if ruid != '' and ruid ==uid:
                        rr['result'] = result
                        break

        return rr

############################################################################


def flatten_dict(d, flat_dict= {}, prefix = ''):

    for k in d:
        v = d[k]

        if type(v) is dict:
            flatten_dict(v, flat_dict, prefix + k + '.')
        else:
            flat_dict[prefix + k] = v

    return flat_dict



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def preprocess(i):
    os_info = i['os_info']

    env = i['env']
    env['+ CFLAGS'] = []
    env['+ CXXFLAGS'] = []
    env['+ FFLAGS'] = []
    env['+ LDFLAGS'] = []

    # TBD: add unified flags for Windows
    if os_info['platform'] == 'windows':
        return {'return': 0}

    if env.get("CM_FAST_COMPILATION") in ["yes", "on", "1"]:
        DEFAULT_COMPILER_FLAGS = env.get("CM_COMPILER_FLAGS_FAST", "-O3")
        # -flto") - this flag is not always available
        DEFAULT_LINKER_FLAGS = env.get("CM_LINKER_FLAGS_FAST", "-O3")
    elif env.get("CM_DEBUG_COMPILATION") in ["yes", "on", "1"]:
        DEFAULT_COMPILER_FLAGS = env.get("CM_COMPILER_FLAGS_DEBUG", "-O0")
        DEFAULT_LINKER_FLAGS = env.get("CM_LINKER_FLAGS_DEBUG", "-O0")
    else:
        DEFAULT_COMPILER_FLAGS = env.get("CM_COMPILER_FLAGS_DEFAULT", "-O2")
        DEFAULT_LINKER_FLAGS = env.get("CM_LINKER_FLAGS_DEFAULT", "-O2")

    env['+ CFLAGS'] += DEFAULT_COMPILER_FLAGS.split(" ")
    env['+ CXXFLAGS'] += DEFAULT_COMPILER_FLAGS.split(" ")
    env['+ FFLAGS'] += DEFAULT_COMPILER_FLAGS.split(" ")
    env['+ LDFLAGS'] += DEFAULT_LINKER_FLAGS.split(" ")

    env['+ CFLAGS'] = list(set(env['+ CFLAGS']))
    env['+ CXXFLAGS'] = list(set(env['+ CXXFLAGS']))
    env['+ FFLAGS'] = list(set(env['+ FFLAGS']))
    env['+ LDFLAGS'] = list(set(env['+ LDFLAGS']))

    sys_cmd = "cpp -v /dev/null -o /dev/null 2>&1"
    result = subprocess.check_output(sys_cmd, shell=True).decode("utf-8")
    start = False
    inc_dir = []
    for out in result.split("\n"):
        if "> search starts here" not in out and not start:
            continue
        if not start:
            start = True
            continue
        if "End of search list" in out:
            break
        if 'gcc' not in out:
            inc_dir.append(out.strip())
    env['+CM_HOST_OS_DEFAULT_INCLUDE_PATH'] = inc_dir

#    if env['CM_C_COMPILER_BIN'] == 'icc':
#        if env['CM_CPUINFO_Vendor_ID'] == 'GenuineIntel':
#            if int(env['CM_CPUINFO_CPU_family']) >= 0:
#                env['+ CFLAGS'] += ["-ipo"]
#    if env['CM_C_COMPILER_BIN'] == 'gcc':
#        if env['CM_HOST_CPU_VENDOR_ID'] == 'AMD':
#            if int(env['CM_HOST_CPU_FAMILY']) >= 0:
#                env['+ CFLAGS'] += ["-march=znver2", "-flto"]

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    need_version = env.get('CM_VERSION', '')
    clang_file_name = "clang"
    if need_version == '':
        return {'return': 1,
                'error': 'internal problem - CM_VERSION is not defined in env'}

    print(recursion_spaces + '    # Requested version: {}'.format(need_version))

    host_os_bits = env['CM_HOST_OS_BITS']

    if os_info['platform'] != 'windows':
        host_os_machine = env['CM_HOST_OS_MACHINE']  # ABI

    # Prepare package name
    # First check if it is forced by external environment
    package_name = env.get('CM_LLVM_PACKAGE', '').strip()
    if package_name == '':
        need_version_split = need_version.split('.')

        # If package_name if not forced, attempt to synthesize it based on OS
        # and arch
        if os_info['platform'] == 'darwin':
            force_arch = env.get(
                'CM_LLVM_PACKAGE_FORCE_ARCH',
                '')  # To allow x86_64 if needed
            if force_arch == '':
                force_arch = 'arm64'
            force_darwin_version = env.get(
                'CM_LLVM_PACKAGE_FORCE_DARWIN_VERSION', '')
            if force_darwin_version == '':
                if len(need_version_split) > 0:
                    hver = 0
                    try:
                        hver = int(need_version_split[0])
                    except BaseException:
                        pass

                    if hver > 0 and hver < 16:
                        force_darwin_version = '21.0'
                    else:
                        force_darwin_version = '22.0'
            package_name = 'clang+llvm-' + need_version + '-' + force_arch + \
                '-apple-darwin' + force_darwin_version + '.tar.xz'

        elif os_info['platform'] == 'windows':
            package_name = 'LLVM-' + need_version + '-win' + host_os_bits + '.exe'
            clang_file_name = "clang.exe"

            print('')
            print('WARNING: Please copy the following path and then paste it')
            print('         when LLVM installer asks you about the "Destination Folder":')
            print('')
            print(os.getcwd())
            print('')
            input('Press Enter to continue!')

        else:
            if host_os_machine.startswith(
                    'arm') or host_os_machine.startswith('aarch'):
                if host_os_bits == '64':
                    package_name = 'clang+llvm-' + need_version + '-aarch64-linux-gnu.tar.xz'
                else:
                    package_name = 'clang+llvm-' + need_version + '-armv7a-linux-gnueabihf.tar.xz'
            else:
                host_os_flavor = env['CM_HOST_OS_FLAVOR']

                host_os_version = env['CM_HOST_OS_VERSION']

#              if 'debian' in host_os_flavor:
#                  return {'return':1, 'error':'debian is not supported yet'}
#
#              else:
                # Treat all Linux flavours as Ubuntu for now ...

                if True:
                    default_os = '22.04'

                    if len(need_version_split) > 0:
                        hver = 0
                        try:
                            hver = int(need_version_split[0])
                        except BaseException:
                            pass

                        if hver > 0:
                            if hver < 16:
                                default_os = '18.04'
                            else:
                                default_os = '22.04'

                    if need_version == '10.0.1':
                        default_os = '16.04'

                    elif need_version == '11.0.0':
                        default_os = '20.04'

                    elif need_version == '11.0.1':
                        default_os = '16.04'
                        if host_os_version == '20.10':
                            default_os = '20.10'

                    elif need_version == '12.0.0':
                        default_os = '16.04'
                        if host_os_version == '20.04' or host_os_version == '20.10':
                            default_os = '20.04'

                    elif need_version == '12.0.1':
                        default_os = '16.04'
                        # if host_os_version.startswith('18') or host_os_version.startswith('20'):
                        #   default_os = '18.04'

                    elif need_version == '13.0.0':
                        default_os = '16.04'
                        if host_os_version.startswith('20'):
                            default_os = '20.04'

                    elif need_version == '13.0.1':
                        default_os = '18.04'

                    elif need_version == '14.0.0':
                        default_os = '18.04'

                    elif need_version == '15.0.6':
                        default_os = '18.04'

                    elif need_version == '16.0.0':
                        default_os = '18.04'

                    elif need_version == '16.0.4':
                        default_os = '22.04'

                    elif need_version == '17.0.2':
                        default_os = '22.04'

                    elif need_version == '17.0.2':
                        default_os = '22.04'

                    elif need_version == '17.0.4':
                        default_os = '22.04'

                    elif need_version == '17.0.5':
                        default_os = '22.04'

                    elif need_version == '17.0.6':
                        default_os = '22.04'

                package_name = 'clang+llvm-' + need_version + \
                    '-x86_64-linux-gnu-ubuntu-' + default_os + '.tar.xz'

    package_url = 'https://github.com/llvm/llvm-project/releases/download/llvmorg-' + \
        need_version + '/' + package_name

    print(recursion_spaces + '    # Prepared package URL: {}'.format(package_url))

    print('')
    print('Downloading from {} ...'.format(package_url))

    cm = automation.cmind

    r = cm.access({'action': 'download_file',
                   'automation': 'utils,dc2743f8450541e3',
                   'url': package_url})
    if r['return'] > 0:
        return r

    # 'clang+llvm-12.0.0-x86_64-linux-gnu-ubuntu-20.04.tar.xz' # f['filename']
    filename = r['filename']

    env['CM_LLVM_PACKAGE'] = filename
    env['CM_LLVM_INSTALLED_PATH'] = os.path.join(os.getcwd(), 'bin')
    env['CM_LLVM_CLANG_BIN_WITH_PATH'] = os.path.join(
        os.getcwd(), 'bin', clang_file_name)
    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_LLVM_CLANG_BIN_WITH_PATH']

    # We don't need to check default paths here because we force install to
    # cache
    env['+PATH'] = [env['CM_LLVM_INSTALLED_PATH']]

    path_include = os.path.join(os.getcwd(), 'include')
    if os.path.isdir(path_include):
        env['+C_INCLUDE_PATH'] = [path_include]

    return {'return': 0}


def postprocess(i):

    env = i['env']
    version = env['CM_VERSION']
    os_info = i['os_info']

#    cur_dir = os.getcwd()
#    cur_dir_include = os.path.join(cur_dir, 'include')

#    if os.path.isdir(cur_dir_include):
#        if os_info['platform'] == 'darwin':
#           if '+C_INCLUDE_PATH' not in env:
#               env['+C_INCLUDE_PATH'] = []
#           if cur_dir_include not in env['+C_INCLUDE_PATH']:
#               env['+C_INCLUDE_PATH'].append(cur_dir_include)
#
#           if '+CPLUS_INCLUDE_PATH' not in env:
#               env['+CPLUS_INCLUDE_PATH'] = []
#           if cur_dir_include not in env['+CPLUS_INCLUDE_PATH']:
#               env['+CPLUS_INCLUDE_PATH'].append(cur_dir_include)

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    cmd_args = ""

    model_name = env.get("CM_VLLM_SERVER_MODEL_NAME", False)
    if not model_name:
        return {'return': 1, 'error': 'Model name not specified'}
    else:
        cmd_args += f" --model {env['CM_ML_MODEL_PATH']} --served-model-name {model_name}"

    tp_size = env.get("CM_VLLM_SERVER_TP_SIZE", False)
    if tp_size:
        cmd_args += f" --tensor-parallel-size {tp_size}"

    pp_size = env.get("CM_VLLM_SERVER_PP_SIZE", False)
    if pp_size:
        cmd_args += f" --pipeline-parallel-size {pp_size}"

    api_key = env.get("CM_VLLM_SERVER_API_KEY", "root")
    if pp_size:
        cmd_args += f" --api-key {api_key}"

    distributed_executor_backend = env.get(
        "CM_VLLM_SERVER_DIST_EXEC_BACKEND", False)
    if distributed_executor_backend:
        cmd_args += f" --distributed-executor-backend {distributed_executor_backend}"

    host = env.get("CM_VLLM_SERVER_HOST", False)
    if host:
        cmd_args += f" --host {host}"

    port = env.get("CM_VLLM_SERVER_PORT", False)
    if port:
        cmd_args += f" --port {port}"

    uvicorn_log_level = env.get("CM_VLLM_SERVER_UVICORN_LOG_LEVEL", False)
    if uvicorn_log_level:
        cmd_args += f" --uvicorn-log-level {uvicorn_log_level}"

    allow_credentials = env.get("CM_VLLM_SERVER_ALLOW_CREDENTIALS", False)
    if allow_credentials:
        cmd_args += f" --allow-credentials"

    allowed_origins = env.get("CM_VLLM_SERVER_ALLOWED_ORIGINS", False)
    if allowed_origins:
        cmd_args += f" --allowed-origins {allowed_origins}"

    allowed_methods = env.get("CM_VLLM_SERVER_ALLOWED_METHODS", False)
    if allowed_methods:
        cmd_args += f" --allowed-methods {allowed_methods}"

    allowed_headers = env.get("CM_VLLM_SERVER_ALLOWED_HEADERS", False)
    if allowed_headers:
        cmd_args += f" --allowed-headers {allowed_headers}"

    lora_modules = env.get("CM_VLLM_SERVER_LORA_MODULES", False)
    if lora_modules:
        cmd_args += f" --lora-modules {lora_modules}"

    prompt_adapters = env.get("CM_VLLM_SERVER_PROMPT_ADAPTERS", False)
    if prompt_adapters:
        cmd_args += f" --prompt-adapters {prompt_adapters}"

    chat_template = env.get("CM_VLLM_SERVER_CHAT_TEMPLATE", False)
    if chat_template:
        cmd_args += f" --chat-template {chat_template}"

    response_role = env.get("CM_VLLM_SERVER_RESPONSE_ROLE", False)
    if response_role:
        cmd_args += f" --response-role {response_role}"

    ssl_keyfile = env.get("CM_VLLM_SERVER_SSL_KEYFILE", False)
    if ssl_keyfile:
        cmd_args += f" --ssl-keyfile {ssl_keyfile}"

    ssl_certfile = env.get("CM_VLLM_SERVER_SSL_CERTFILE", False)
    if ssl_certfile:
        cmd_args += f" --ssl-certfile {ssl_certfile}"

    ssl_ca_certs = env.get("CM_VLLM_SERVER_SSL_CA_CERTS", False)
    if ssl_ca_certs:
        cmd_args += f" --ssl-ca-certs {ssl_ca_certs}"

    ssl_cert_reqs = env.get("CM_VLLM_SERVER_SSL_CERT_REQS", False)
    if ssl_cert_reqs:
        cmd_args += f" --ssl-cert-reqs {ssl_cert_reqs}"

    root_path = env.get("CM_VLLM_SERVER_ROOT_PATH", False)
    if root_path:
        cmd_args += f" --root-path {root_path}"

    middleware = env.get("CM_VLLM_SERVER_MIDDLEWARE", False)
    if middleware:
        cmd_args += f" --middleware {middleware}"

    tokenizer = env.get("CM_VLLM_SERVER_TOKENIZER", False)
    if tokenizer:
        cmd_args += f" --tokenizer {tokenizer}"

    skip_tokenizer_init = env.get("CM_VLLM_SERVER_SKIP_TOKENIZER_INIT", False)
    if skip_tokenizer_init:
        cmd_args += f" --skip-tokenizer-init"

    revision = env.get("CM_VLLM_SERVER_REVISION", False)
    if revision:
        cmd_args += f" --revision {revision}"

    code_revision = env.get("CM_VLLM_SERVER_CODE_REVISION", False)
    if code_revision:
        cmd_args += f" --code-revision {code_revision}"

    tokenizer_revision = env.get("CM_VLLM_SERVER_TOKENIZER_REVISION", False)
    if tokenizer_revision:
        cmd_args += f" --tokenizer-revision {tokenizer_revision}"

    tokenizer_mode = env.get("CM_VLLM_SERVER_TOKENIZER_MODE", False)
    if tokenizer_mode:
        cmd_args += f" --tokenizer-mode {tokenizer_mode}"

    trust_remote_code = env.get("CM_VLLM_SERVER_TRUST_REMOTE_CODE", False)
    if trust_remote_code:
        cmd_args += f" --trust-remote-code"

    download_dir = env.get("CM_VLLM_SERVER_DOWNLOAD_DIR", False)
    if download_dir:
        cmd_args += f" --download-dir {download_dir}"

    load_format = env.get("CM_VLLM_SERVER_LOAD_FORMAT", False)
    if load_format:
        cmd_args += f" --load-format {load_format}"

    dtype = env.get("CM_VLLM_SERVER_DTYPE", False)
    if dtype:
        cmd_args += f" --dtype {dtype}"

    kv_cache_dtype = env.get("CM_VLLM_SERVER_KV_CACHE_DTYPE", False)
    if kv_cache_dtype:
        cmd_args += f" --kv-cache-dtype {kv_cache_dtype}"

    quantization_param_path = env.get(
        "CM_VLLM_SERVER_QUANTIZATION_PARAM_PATH", False)
    if quantization_param_path:
        cmd_args += f" --quantization-param-path {quantization_param_path}"

    max_model_len = env.get("CM_VLLM_SERVER_MAX_MODEL_LEN", False)
    if max_model_len:
        cmd_args += f" --max-model-len {max_model_len}"

    guided_decoding_backend = env.get(
        "CM_VLLM_SERVER_GUIDED_DECODING_BACKEND", False)
    if guided_decoding_backend:
        cmd_args += f" --guided-decoding-backend {guided_decoding_backend}"

    worker_use_ray = env.get("CM_VLLM_SERVER_WORKER_USE_RAY", False)
    if worker_use_ray:
        cmd_args += f" --worker-use-ray"

    max_parallel_loading_workers = env.get(
        "CM_VLLM_SERVER_MAX_PARALLEL_LOADING_WORKERS", False)
    if max_parallel_loading_workers:
        cmd_args += f" --max-parallel-loading-workers {max_parallel_loading_workers}"

    ray_workers_use_nsight = env.get(
        "CM_VLLM_SERVER_RAY_WORKERS_USE_NSIGHT", False)
    if ray_workers_use_nsight:
        cmd_args += f" --ray-workers-use-nsight"

    block_size = env.get("CM_VLLM_SERVER_BLOCK_SIZE", False)
    if block_size:
        cmd_args += f" --block-size {block_size}"

    enable_prefix_caching = env.get(
        "CM_VLLM_SERVER_ENABLE_PREFIX_CACHING", False)
    if enable_prefix_caching:
        cmd_args += f" --enable-prefix-caching"

    disable_sliding_window = env.get(
        "CM_VLLM_SERVER_DISABLE_SLIDING_WINDOW", False)
    if disable_sliding_window:
        cmd_args += f" --disable-sliding-window"

    use_v2_block_manager = env.get(
        "CM_VLLM_SERVER_USE_V2_BLOCK_MANAGER", False)
    if use_v2_block_manager:
        cmd_args += f" --use-v2-block-manager"

    num_lookahead_slots = env.get("CM_VLLM_SERVER_NUM_LOOKAHEAD_SLOTS", False)
    if num_lookahead_slots:
        cmd_args += f" --num-lookahead-slots {num_lookahead_slots}"

    seed = env.get("CM_VLLM_SERVER_SEED", False)
    if seed:
        cmd_args += f" --seed {seed}"

    swap_space = env.get("CM_VLLM_SERVER_SWAP_SPACE", False)
    if swap_space:
        cmd_args += f" --swap-space {swap_space}"

    gpu_memory_utilization = env.get(
        "CM_VLLM_SERVER_GPU_MEMORY_UTILIZATION", False)
    if gpu_memory_utilization:
        cmd_args += f" --gpu-memory-utilization {gpu_memory_utilization}"

    num_gpu_blocks_override = env.get(
        "CM_VLLM_SERVER_NUM_GPU_BLOCKS_OVERRIDE", False)
    if num_gpu_blocks_override:
        cmd_args += f" --num-gpu-blocks-override {num_gpu_blocks_override}"

    max_num_batched_tokens = env.get(
        "CM_VLLM_SERVER_MAX_NUM_BATCHED_TOKENS", False)
    if max_num_batched_tokens:
        cmd_args += f" --max-num-batched-tokens {max_num_batched_tokens}"

    max_num_seqs = env.get("CM_VLLM_SERVER_MAX_NUM_SEQS", False)
    if max_num_seqs:
        cmd_args += f" --max-num-seqs {max_num_seqs}"

    max_logprobs = env.get("CM_VLLM_SERVER_MAX_LOGPROBS", False)
    if max_logprobs:
        cmd_args += f" --max-logprobs {max_logprobs}"

    disable_log_stats = env.get("CM_VLLM_SERVER_DISABLE_LOG_STATS", False)
    if disable_log_stats:
        cmd_args += f" --disable-log-stats"

    quantization = env.get("CM_VLLM_SERVER_QUANTIZATION", False)
    if quantization:
        cmd_args += f" --quantization {quantization}"

    rope_scaling = env.get("CM_VLLM_SERVER_ROPE_SCALING", False)
    if rope_scaling:
        cmd_args += f" --rope-scaling {rope_scaling}"

    rope_theta = env.get("CM_VLLM_SERVER_ROPE_THETA", False)
    if rope_theta:
        cmd_args += f" --rope-theta {rope_theta}"

    enforce_eager = env.get("CM_VLLM_SERVER_ENFORCE_EAGER", False)
    if enforce_eager:
        cmd_args += f" --enforce-eager"

    max_context_len_to_capture = env.get(
        "CM_VLLM_SERVER_MAX_CONTEXT_LEN_TO_CAPTURE", False)
    if max_context_len_to_capture:
        cmd_args += f" --max-context-len-to-capture {max_context_len_to_capture}"

    max_seq_len_to_capture = env.get(
        "CM_VLLM_SERVER_MAX_SEQ_LEN_TO_CAPTURE", False)
    if max_seq_len_to_capture:
        cmd_args += f" --max-seq-len-to-capture {max_seq_len_to_capture}"

    disable_custom_all_reduce = env.get(
        "CM_VLLM_SERVER_DISABLE_CUSTOM_ALL_REDUCE", False)
    if disable_custom_all_reduce:
        cmd_args += f" --disable-custom-all-reduce"

    tokenizer_pool_size = env.get("CM_VLLM_SERVER_TOKENIZER_POOL_SIZE", False)
    if tokenizer_pool_size:
        cmd_args += f" --tokenizer-pool-size {tokenizer_pool_size}"

    tokenizer_pool_type = env.get("CM_VLLM_SERVER_TOKENIZER_POOL_TYPE", False)
    if tokenizer_pool_type:
        cmd_args += f" --tokenizer-pool-type {tokenizer_pool_type}"

    tokenizer_pool_extra_config = env.get(
        "CM_VLLM_SERVER_TOKENIZER_POOL_EXTRA_CONFIG", False)
    if tokenizer_pool_extra_config:
        cmd_args += f" --tokenizer-pool-extra-config {tokenizer_pool_extra_config}"

    enable_lora = env.get("CM_VLLM_SERVER_ENABLE_LORA", False)
    if enable_lora:
        cmd_args += f" --enable-lora"

    max_loras = env.get("CM_VLLM_SERVER_MAX_LORAS", False)
    if max_loras:
        cmd_args += f" --max-loras {max_loras}"

    max_lora_rank = env.get("CM_VLLM_SERVER_MAX_LORA_RANK", False)
    if max_lora_rank:
        cmd_args += f" --max-lora-rank {max_lora_rank}"

    lora_extra_vocab_size = env.get(
        "CM_VLLM_SERVER_LORA_EXTRA_VOCAB_SIZE", False)
    if lora_extra_vocab_size:
        cmd_args += f" --lora-extra-vocab-size {lora_extra_vocab_size}"

    lora_dtype = env.get("CM_VLLM_SERVER_LORA_DTYPE", False)
    if lora_dtype:
        cmd_args += f" --lora-dtype {lora_dtype}"

    long_lora_scaling_factors = env.get(
        "CM_VLLM_SERVER_LONG_LORA_SCALING_FACTORS", False)
    if long_lora_scaling_factors:
        cmd_args += f" --long-lora-scaling-factors {long_lora_scaling_factors}"

    max_cpu_loras = env.get("CM_VLLM_SERVER_MAX_CPU_LORAS", False)
    if max_cpu_loras:
        cmd_args += f" --max-cpu-loras {max_cpu_loras}"

    fully_sharded_loras = env.get("CM_VLLM_SERVER_FULLY_SHARDED_LORAS", False)
    if fully_sharded_loras:
        cmd_args += f" --fully-sharded-loras"

    enable_prompt_adapter = env.get(
        "CM_VLLM_SERVER_ENABLE_PROMPT_ADAPTER", False)
    if enable_prompt_adapter:
        cmd_args += f" --enable-prompt-adapter"

    max_prompt_adapters = env.get("CM_VLLM_SERVER_MAX_PROMPT_ADAPTERS", False)
    if max_prompt_adapters:
        cmd_args += f" --max-prompt-adapters {max_prompt_adapters}"

    max_prompt_adapter_token = env.get(
        "CM_VLLM_SERVER_MAX_PROMPT_ADAPTER_TOKEN", False)
    if max_prompt_adapter_token:
        cmd_args += f" --max-prompt-adapter-token {max_prompt_adapter_token}"

    device = env.get("CM_VLLM_SERVER_DEVICE", False)
    if device:
        cmd_args += f" --device {device}"

    scheduler_delay_factor = env.get(
        "CM_VLLM_SERVER_SCHEDULER_DELAY_FACTOR", False)
    if scheduler_delay_factor:
        cmd_args += f" --scheduler-delay-factor {scheduler_delay_factor}"

    enable_chunked_prefill = env.get(
        "CM_VLLM_SERVER_ENABLE_CHUNKED_PREFILL", False)
    if enable_chunked_prefill:
        cmd_args += f" --enable-chunked-prefill"

    speculative_model = env.get("CM_VLLM_SERVER_SPECULATIVE_MODEL", False)
    if speculative_model:
        cmd_args += f" --speculative-model {speculative_model}"

    num_speculative_tokens = env.get(
        "CM_VLLM_SERVER_NUM_SPECULATIVE_TOKENS", False)
    if num_speculative_tokens:
        cmd_args += f" --num-speculative-tokens {num_speculative_tokens}"

    speculative_draft_tensor_parallel_size = env.get(
        "CM_VLLM_SERVER_SPECULATIVE_DRAFT_TENSOR_PARALLEL_SIZE", False)
    if speculative_draft_tensor_parallel_size:
        cmd_args += f" --speculative-draft-tensor-parallel-size {speculative_draft_tensor_parallel_size}"

    speculative_max_model_len = env.get(
        "CM_VLLM_SERVER_SPECULATIVE_MAX_MODEL_LEN", False)
    if speculative_max_model_len:
        cmd_args += f" --speculative-max-model-len {speculative_max_model_len}"

    speculative_disable_by_batch_size = env.get(
        "CM_VLLM_SERVER_SPECULATIVE_DISABLE_BY_BATCH_SIZE", False)
    if speculative_disable_by_batch_size:
        cmd_args += f" --speculative-disable-by-batch-size {speculative_disable_by_batch_size}"

    ngram_prompt_lookup_max = env.get(
        "CM_VLLM_SERVER_NGRAM_PROMPT_LOOKUP_MAX", False)
    if ngram_prompt_lookup_max:
        cmd_args += f" --ngram-prompt-lookup-max {ngram_prompt_lookup_max}"

    ngram_prompt_lookup_min = env.get(
        "CM_VLLM_SERVER_NGRAM_PROMPT_LOOKUP_MIN", False)
    if ngram_prompt_lookup_min:
        cmd_args += f" --ngram-prompt-lookup-min {ngram_prompt_lookup_min}"

    spec_decoding_acceptance_method = env.get(
        "CM_VLLM_SERVER_SPEC_DECODING_ACCEPTANCE_METHOD", False)
    if spec_decoding_acceptance_method:
        cmd_args += f" --spec-decoding-acceptance-method {spec_decoding_acceptance_method}"

    typical_acceptance_sampler_posterior_threshold = env.get(
        "CM_VLLM_SERVER_TYPICAL_ACCEPTANCE_SAMPLER_POSTERIOR_THRESHOLD", False)
    if typical_acceptance_sampler_posterior_threshold:
        cmd_args += f" --typical-acceptance-sampler-posterior-threshold {typical_acceptance_sampler_posterior_threshold}"

    typical_acceptance_sampler_posterior_alpha = env.get(
        "CM_VLLM_SERVER_TYPICAL_ACCEPTANCE_SAMPLER_POSTERIOR_ALPHA", False)
    if typical_acceptance_sampler_posterior_alpha:
        cmd_args += f" --typical-acceptance-sampler-posterior-alpha {typical_acceptance_sampler_posterior_alpha}"

    model_loader_extra_config = env.get(
        "CM_VLLM_SERVER_MODEL_LOADER_EXTRA_CONFIG", False)
    if model_loader_extra_config:
        cmd_args += f" --model-loader-extra-config {model_loader_extra_config}"

    preemption_mode = env.get("CM_VLLM_SERVER_PREEMPTION_MODE", False)
    if preemption_mode:
        cmd_args += f" --preemption-mode {preemption_mode}"

    served_model_name = env.get("CM_VLLM_SERVER_SERVED_MODEL_NAME", False)
    if served_model_name:
        cmd_args += f" --served-model-name {served_model_name}"

    qlora_adapter_name_or_path = env.get(
        "CM_VLLM_SERVER_QLORA_ADAPTER_NAME_OR_PATH", False)
    if qlora_adapter_name_or_path:
        cmd_args += f" --qlora-adapter-name-or-path {qlora_adapter_name_or_path}"

    otlp_traces_endpoint = env.get(
        "CM_VLLM_SERVER_OTLP_TRACES_ENDPOINT", False)
    if otlp_traces_endpoint:
        cmd_args += f" --otlp-traces-endpoint {otlp_traces_endpoint}"

    engine_use_ray = env.get("CM_VLLM_SERVER_ENGINE_USE_RAY", False)
    if engine_use_ray:
        cmd_args += f" --engine-use-ray"

    disable_log_requests = env.get(
        "CM_VLLM_SERVER_DISABLE_LOG_REQUESTS", False)
    if disable_log_requests:
        cmd_args += f" --disable-log-requests"

    max_log_len = env.get("CM_VLLM_SERVER_MAX_LOG_LEN", False)
    if max_log_len:
        cmd_args += f" --max-log-len {max_log_len}"

    cmd = f"{env['CM_PYTHON_BIN_WITH_PATH']} -m vllm.entrypoints.openai.api_server {cmd_args}"
    print(cmd)

    env['CM_VLLM_RUN_CMD'] = cmd

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    cm = automation.cmind

    path = os.path.dirname(env['CM_ML_MODEL_FILE_WITH_PATH'])

    if env.get("CM_DAE_EXTRACT_DOWNLOADED", " ") != " ":
        env['CM_ML_MODEL_PATH'] = os.path.join(path, env['CM_ML_MODEL_FILE'])
        env['CM_ML_MODEL_FILE_WITH_PATH'] = env['CM_ML_MODEL_PATH']
    else:
        env['CM_ML_MODEL_PATH'] = path

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    clang_file_name = "clang"
    extra_cmake_options = ''

    install_prefix = os.path.join(os.getcwd(), "install")

    if env.get('CM_LLVM_CONDA_ENV', '') == "yes":
        install_prefix = env['CM_CONDA_PREFIX']
        extra_cmake_options = f"-DCMAKE_SHARED_LINKER_FLAGS=-L{install_prefix} -Wl,-rpath,{install_prefix}"

    if env.get('CM_LLVM_16_INTEL_MLPERF_INFERENCE', '') == "yes":
        env['CM_REQUIRE_INSTALL'] = 'yes'
        i['run_script_input']['script_name'] = "install-llvm-16-intel-mlperf-inference"
        clang_file_name = "llvm-link"
        # env['USE_LLVM'] = install_prefix
        # env['LLVM_DIR'] = os.path.join(env['USE_LLVM'], "lib", "cmake", "llvm")
    else:
        if env.get('CM_LLVM_ENABLE_RUNTIMES', '') != '':
            enable_runtimes = env['CM_LLVM_ENABLE_RUNTIMES'].replace(":", ";")
        else:
            enable_runtimes = ''

        if env.get('CM_LLVM_ENABLE_PROJECTS', '') != '':
            enable_projects = env['CM_LLVM_ENABLE_PROJECTS'].replace(":", ";")
        else:
            enable_projects = ''

        llvm_build_type = env['CM_LLVM_BUILD_TYPE']

        cmake_cmd = "cmake " + os.path.join(env["CM_LLVM_SRC_REPO_PATH"], "llvm") + " -GNinja -DCMAKE_BUILD_TYPE=" + llvm_build_type + " -DLLVM_ENABLE_PROJECTS=" + enable_projects + " -DLLVM_ENABLE_RUNTIMES='" + \
            enable_runtimes + "' -DCMAKE_INSTALL_PREFIX=" + install_prefix + \
            " -DLLVM_ENABLE_RTTI=ON  -DLLVM_INSTALL_UTILS=ON -DLLVM_TARGETS_TO_BUILD=X86 " + \
            extra_cmake_options

        env['CM_LLVM_CMAKE_CMD'] = cmake_cmd

    need_version = env.get('CM_VERSION', '')

    # print(cmake_cmd)

    env['CM_LLVM_INSTALLED_PATH'] = install_prefix
    env['CM_LLVM_CLANG_BIN_WITH_PATH'] = os.path.join(
        env['CM_LLVM_INSTALLED_PATH'], "bin", clang_file_name)

    # env['+PATH'] = []
    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_LLVM_CLANG_BIN_WITH_PATH']

    if env.get('CM_LLVM_CONDA_ENV', '') != "yes":
        # We don't need to check default paths here because we force install to
        # cache
        env['+PATH'] = [os.path.join(env['CM_LLVM_INSTALLED_PATH'], "bin")]

        path_include = os.path.join(env['CM_LLVM_INSTALLED_PATH'], 'include')
        if os.path.isdir(path_include):
            env['+C_INCLUDE_PATH'] = [path_include]

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    print("")
    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")
    print("")

    if os_info['platform'] == 'windows':
        MLPERF_CLASSES = ['Airplane', 'Antelope', 'Apple', 'Backpack', 'Balloon', 'Banana',
                          'Barrel', 'Baseball bat', 'Baseball glove', 'Bee', 'Beer', 'Bench', 'Bicycle',
                          'Bicycle helmet', 'Bicycle wheel', 'Billboard', 'Book', 'Bookcase', 'Boot',
                          'Bottle', 'Bowl', 'Bowling equipment', 'Box', 'Boy', 'Brassiere', 'Bread',
                          'Broccoli', 'Bronze sculpture', 'Bull', 'Bus', 'Bust', 'Butterfly', 'Cabinetry',
                          'Cake', 'Camel', 'Camera', 'Candle', 'Candy', 'Cannon', 'Canoe', 'Carrot', 'Cart',
                          'Castle', 'Cat', 'Cattle', 'Cello', 'Chair', 'Cheese', 'Chest of drawers', 'Chicken',
                          'Christmas tree', 'Coat', 'Cocktail', 'Coffee', 'Coffee cup', 'Coffee table', 'Coin',
                          'Common sunflower', 'Computer keyboard', 'Computer monitor', 'Convenience store',
                          'Cookie', 'Countertop', 'Cowboy hat', 'Crab', 'Crocodile', 'Cucumber', 'Cupboard',
                          'Curtain', 'Deer', 'Desk', 'Dinosaur', 'Dog', 'Doll', 'Dolphin', 'Door', 'Dragonfly',
                          'Drawer', 'Dress', 'Drum', 'Duck', 'Eagle', 'Earrings', 'Egg (Food)', 'Elephant',
                          'Falcon', 'Fedora', 'Flag', 'Flowerpot', 'Football', 'Football helmet', 'Fork',
                          'Fountain', 'French fries', 'French horn', 'Frog', 'Giraffe', 'Girl', 'Glasses',
                          'Goat', 'Goggles', 'Goldfish', 'Gondola', 'Goose', 'Grape', 'Grapefruit', 'Guitar',
                          'Hamburger', 'Handbag', 'Harbor seal', 'Headphones', 'Helicopter', 'High heels',
                          'Hiking equipment', 'Horse', 'House', 'Houseplant', 'Human arm', 'Human beard',
                          'Human body', 'Human ear', 'Human eye', 'Human face', 'Human foot', 'Human hair',
                          'Human hand', 'Human head', 'Human leg', 'Human mouth', 'Human nose', 'Ice cream',
                          'Jacket', 'Jeans', 'Jellyfish', 'Juice', 'Kitchen & dining room table', 'Kite',
                          'Lamp', 'Lantern', 'Laptop', 'Lavender (Plant)', 'Lemon', 'Light bulb', 'Lighthouse',
                          'Lily', 'Lion', 'Lipstick', 'Lizard', 'Man', 'Maple', 'Microphone', 'Mirror',
                          'Mixing bowl', 'Mobile phone', 'Monkey', 'Motorcycle', 'Muffin', 'Mug', 'Mule',
                          'Mushroom', 'Musical keyboard', 'Necklace', 'Nightstand', 'Office building',
                          'Orange', 'Owl', 'Oyster', 'Paddle', 'Palm tree', 'Parachute', 'Parrot', 'Pen',
                          'Penguin', 'Personal flotation device', 'Piano', 'Picture frame', 'Pig', 'Pillow',
                          'Pizza', 'Plate', 'Platter', 'Porch', 'Poster', 'Pumpkin', 'Rabbit', 'Rifle',
                          'Roller skates', 'Rose', 'Salad', 'Sandal', 'Saucer', 'Saxophone', 'Scarf', 'Sea lion',
                          'Sea turtle', 'Sheep', 'Shelf', 'Shirt', 'Shorts', 'Shrimp', 'Sink', 'Skateboard',
                          'Ski', 'Skull', 'Skyscraper', 'Snake', 'Sock', 'Sofa bed', 'Sparrow', 'Spider', 'Spoon',
                          'Sports uniform', 'Squirrel', 'Stairs', 'Stool', 'Strawberry', 'Street light',
                          'Studio couch', 'Suit', 'Sun hat', 'Sunglasses', 'Surfboard', 'Sushi', 'Swan',
                          'Swimming pool', 'Swimwear', 'Tank', 'Tap', 'Taxi', 'Tea', 'Teddy bear', 'Television',
                          'Tent', 'Tie', 'Tiger', 'Tin can', 'Tire', 'Toilet', 'Tomato', 'Tortoise', 'Tower',
                          'Traffic light', 'Train', 'Tripod', 'Truck', 'Trumpet', 'Umbrella', 'Van', 'Vase',
                          'Vehicle registration plate', 'Violin', 'Wall clock', 'Waste container', 'Watch',
                          'Whale', 'Wheel', 'Wheelchair', 'Whiteboard', 'Window', 'Wine', 'Wine glass', 'Woman',
                          'Zebra', 'Zucchini']

        x = ''
        for v in MLPERF_CLASSES:
            if x != '':
                x += ' '
            x += '"' + v + '"'
        env['CM_DATASET_OPENIMAGES_CLASSES'] = x

    return {'return': 0}


def postprocess(i):
    env = i['env']

    env['CM_DATASET_ANNOTATIONS_DIR_PATH'] = os.path.join(
        os.getcwd(), 'install', 'annotations')

    if env.get('CM_DATASET_CALIBRATION', '') == "no":
        env['CM_DATASET_PATH_ROOT'] = os.path.join(os.getcwd(), 'install')
        env['CM_DATASET_PATH'] = os.path.join(
            os.getcwd(), 'install', 'validation', 'data')
        annotations_file_path = os.path.join(
            env['CM_DATASET_ANNOTATIONS_DIR_PATH'],
            "openimages-mlperf.json")
        env['CM_DATASET_VALIDATION_ANNOTATIONS_FILE_PATH'] = annotations_file_path
        env['CM_DATASET_ANNOTATIONS_FILE_PATH'] = annotations_file_path
        env['CM_DATASET_OPENIMAGES_VALIDATION_ANNOTATIONS_FILE_PATH'] = annotations_file_path
        if env.get("CM_DATASET_OPENIMAGES_CUSTOM_ANNOTATIONS", '') == "yes":
            annotations_file_src = env['CM_DATASET_OPENIMAGES_ANNOTATIONS_FILE_PATH']
            shutil.copy(
                annotations_file_src,
                env['CM_DATASET_ANNOTATIONS_DIR_PATH'])
        env['CM_DATASET_OPENIMAGES_PATH'] = env['CM_DATASET_PATH']
        env['CM_DATASET_OPENIMAGES_PATH_ROOT'] = env['CM_DATASET_PATH_ROOT']
    else:
        env['CM_CALIBRATION_DATASET_PATH'] = os.path.join(
            os.getcwd(), 'install', 'calibration', 'data')
        env['CM_OPENIMAGES_CALIBRATION_DATASET_PATH'] = os.path.join(
            os.getcwd(), 'install', 'calibration', 'data')
        env['CM_CALIBRATION_DATASET_PATH_ROOT'] = os.path.join(
            os.getcwd(), 'install')
        annotations_file_path = os.path.join(
            env['CM_DATASET_ANNOTATIONS_DIR_PATH'],
            "openimages-calibration-mlperf.json")
        env['CM_DATASET_CALIBRATION_ANNOTATIONS_FILE_PATH'] = annotations_file_path

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    dlrm_data_path = env.get(
        'CM_DLRM_DATA_PATH', env.get(
            'DLRM_DATA_PATH', ''))
    if dlrm_data_path == '':
        print(
            f'Data path is not given as input through --dlrm_data_path. Using the cache directory:{os.getcwd()} as the data path')
        dlrm_data_path = os.getcwd()
    elif not os.path.exists(dlrm_data_path):
        return {'return': 1, 'error': "given dlrm data path does not exists"}

    # creating required folders inside the dlrm data path if not exists
    # criteo dataset
    criteo_fp32_path = os.path.join(dlrm_data_path, "criteo", "day23", "fp32")
    if not os.path.exists(criteo_fp32_path):
        os.makedirs(criteo_fp32_path)

    # dlrm model
    model_path = os.path.join(dlrm_data_path, "model")
    if not os.path.exists(model_path):
        os.makedirs(model_path)

    meta = i['meta']

    script_path = i['run_script_input']['path']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    variation = env['CM_DLRM_DATA_VARIATION']

    if variation == "nvidia":
        if not os.path.exists(os.path.join(dlrm_data_path, "model")):
            print(f'model directory is missing inside {dlrm_data_path}')
            env['CM_DLRM_MODEL_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(dlrm_data_path, "criteo")):
            print(f'criteo directory is missing inside {dlrm_data_path}')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(
                dlrm_data_path, "model", "model_weights")):
            print(
                f'model_weights directory is missing inside {dlrm_data_path}/model')
            env['CM_DLRM_MODEL_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(dlrm_data_path, "criteo", "day23")):
            print(f'day23 directory is missing inside {dlrm_data_path}/day23')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(
                dlrm_data_path, "criteo", "day23", "fp32")):
            print(
                f'fp32 directory is missing inside {dlrm_data_path}/criteo/day23')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(dlrm_data_path, "criteo", "day23", "fp32", "day_23_sparse_multi_hot.npz")) and not os.path.exists(
                os.path.join(dlrm_data_path, "criteo", "day23", "fp32", "day_23_sparse_multi_hot_unpacked")):
            print(
                f'day_23_sparse_multi_hot.npz or day_23_sparse_multi_hot_unpacked is missing inside {dlrm_data_path}/criteo/day23/fp32')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(
                dlrm_data_path, "criteo", "day23", "fp32", "day_23_dense.npy")):
            print(
                f'day_23_dense.npy is missing inside {dlrm_data_path}/criteo/day23/fp32')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(
                dlrm_data_path, "criteo", "day23", "fp32", "day_23_labels.npy")):
            print(
                f'day_23_labels.npy is missing inside {dlrm_data_path}/criteo/day23/fp32')
            env['CM_DLRM_DATASET_DOWNLOAD'] = True
        if not os.path.exists(os.path.join(
                dlrm_data_path, "criteo", "day23", "raw_data")):
            if env.get('CM_CRITEO_DAY23_RAW_DATA_PATH', '') == '':
                return {
                    'return': 1, 'error': 'Raw data missing inside {dlrm_data_path}/criteo/day23. Specify the target folder through input mapping(--criteo_day23_raw_data_path="path to raw criteo dataset")'}

        run_cmd = ''
        xsep = ' && '

        # addition of run command to download the datasets and model
        if env.get('CM_DLRM_DATASET_DOWNLOAD', False) == True:
            run_cmd += 'cp -r "$CM_CRITEO_PREPROCESSED_PATH"/. ' + \
                os.path.join(dlrm_data_path, "criteo", "day23", "fp32") + xsep
        if env.get('CM_DLRM_MODEL_DOWNLOAD', False) == True:
            run_cmd += 'cp -r "$CM_ML_MODEL_FILE_WITH_PATH"/. ' + \
                os.path.join(dlrm_data_path, "model") + xsep

        if env.get('CM_DLRM_DATASET_DOWNLOAD', '') != True:
            if not os.path.exists(os.path.join(
                    dlrm_data_path, "criteo", "day23", "fp32", "day_23_sparse_multi_hot_unpacked")):
                os.system(f"unzip {os.path.join(dlrm_data_path, 'criteo', 'day23', 'fp32', 'day_23_sparse_multi_hot.npz')} -d {os.path.join(dlrm_data_path, 'criteo', 'day23', 'fp32', 'day_23_sparse_multi_hot_unpacked')}")
        else:
            run_cmd += f"unzip {os.path.join(dlrm_data_path, 'criteo', 'day23', 'fp32', 'day_23_sparse_multi_hot.npz')} -d {os.path.join(dlrm_data_path, 'criteo', 'day23', 'fp32', 'day_23_sparse_multi_hot_unpacked')}" + xsep

        if os.path.exists(os.path.join(dlrm_data_path, "criteo", "day23", "fp32",
                          "day_23_sparse_multi_hot.npz")) or env['CM_DLRM_DATASET_DOWNLOAD'] == True:
            file_path = os.path.join(
                dlrm_data_path,
                "criteo",
                "day23",
                "fp32",
                "day_23_sparse_multi_hot.npz")
            run_cmd += ("echo {} {} | md5sum -c").format(
                'c46b7e31ec6f2f8768fa60bdfc0f6e40', file_path) + xsep

        file_path = os.path.join(
            dlrm_data_path,
            "criteo",
            "day23",
            "fp32",
            "day_23_dense.npy")
        run_cmd += ("echo {} {} | md5sum -c").format(
            'cdf7af87cbc7e9b468c0be46b1767601', file_path) + xsep

        file_path = os.path.join(
            dlrm_data_path,
            "criteo",
            "day23",
            "fp32",
            "day_23_labels.npy")
        run_cmd += ("echo {} {} | md5sum -c").format(
            'dd68f93301812026ed6f58dfb0757fa7', file_path) + xsep

        dir_path = os.path.join(dlrm_data_path, "criteo", "day23", "fp32")
        run_cmd += ("cd {}; md5sum -c {}").format(dir_path,
                                                  os.path.join(script_path, "checksums.txt"))

        env['CM_DLRM_V2_DAY23_FILE_PATH'] = os.path.join(
            dlrm_data_path, "criteo", "day23", "raw_data")
        env['CM_DLRM_V2_AGGREGATION_TRACE_FILE_PATH'] = os.path.join(
            dlrm_data_path, "criteo", "day23", "sample_partition.txt")

        env['CM_RUN_CMD'] = run_cmd

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env.get('CM_DLRM_DATA_PATH', '') == '' and env.get(
            'DLRM_DATA_PATH', '') == '':
        env['CM_DLRM_DATA_PATH'] = os.getcwd()
    else:
        env['CM_GET_DEPENDENT_CACHED_PATH'] = env.get(
            'CM_DLRM_DATA_PATH', env['DLRM_DATA_PATH'])

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    env = i['env']

    skip_preprocessing = False
    if env.get('CM_DATASET_PREPROCESSED_PATH', '') != '':
        '''
        Path with preprocessed dataset given as input
        '''
        skip_preprocessing = True
        print("Using preprocessed criteo dataset from '" +
              env['CM_DATASET_PREPROCESSED_PATH'] + "'")

    if not skip_preprocessing and env.get(
            'CM_DATASET_PREPROCESSED_OUTPUT_PATH', '') != '':
        env['CM_DATASET_PREPROCESSED_PATH'] = os.getcwd()

    if not skip_preprocessing and env.get(
            'CM_DATASET_CRITEO_MULTIHOT', '') == 'yes':
        i['run_script_input']['script_name'] = "run-multihot"
        # ${CM_PYTHON_BIN_WITH_PATH} ${CM_TMP_CURRENT_SCRIPT_PATH}/preprocess.py
        output_dir = env['CM_DATASET_PREPROCESSED_PATH']
        dataset_path = env['CM_DATASET_PATH']
        tmp_dir = os.path.join(output_dir, "tmp")
        run_dir = os.path.join(
            env['CM_MLPERF_TRAINING_SOURCE'],
            "recommendation_v2",
            "torchrec_dlrm",
            "scripts")
        env['CM_RUN_CMD'] = f'cd {run_dir} && bash ./process_Criteo_1TB_Click_Logs_dataset.sh {dataset_path} {tmp_dir} {output_dir} '

        print("Using MLCommons Training source from '" +
              env['CM_MLPERF_TRAINING_SOURCE'] + "'")

    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_CRITEO_PREPROCESSED_PATH'] = env['CM_DATASET_PREPROCESSED_PATH']

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_CRITEO_PREPROCESSED_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import subprocess
from os.path import exists


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    input_dir = env.get("CM_TAR_INPUT_DIR", "")
    if input_dir == "":
        return {'return': 1, 'error': 'Please set CM_TAR_INPUT_DIR'}
    output_dir = env.get("CM_TAR_OUTPUT_DIR", "")
    if output_dir == "":
        output_dir = os.getcwd()
    output_file = env.get("CM_TAR_OUTFILE", "")
    input_dirname = os.path.basename(input_dir)
    if output_file == "":
        output_file = input_dirname + ".tar.gz"
    from pathlib import Path
    input_path = Path(input_dir)
    cd_dir = input_path.parent.absolute()
    CMD = 'tar --directory ' + \
        str(cd_dir) + ' -czf ' + os.path.join(output_dir,
                                              output_file) + ' ' + input_dirname
    print(CMD)
    ret = os.system(CMD)
    print("Tar file " + os.path.join(output_dir, output_file) + " created")

    return {'return': ret}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    new_dir = env.get('CM_CREATE_PATCH_NEW', '')
    if new_dir == '':
        return {'return': 1, 'error': 'specify NEW directory using --new'}
    if not os.path.isdir(new_dir):
        return {'return': 1,
                'error': 'NEW directory doesn\'t exist {}'.format(new_dir)}

    old_dir = env.get('CM_CREATE_PATCH_OLD', '')
    if old_dir == '':
        return {'return': 1, 'error': 'specify OLD directory using --old'}
    if not os.path.isdir(old_dir):
        return {'return': 1,
                'error': 'OLD directory doesn\'t exist {}'.format(old_dir)}

    exclude = env.get('CM_CREATE_PATCH_EXCLUDE', '').strip()
    x_exclude = ''

    if exclude != '':
        for e in exclude.split(','):
            x_exclude += ' --exclude={}'.format(e)

    cmd = 'diff -Naur {} {} {} > patch.patch'.format(
        x_exclude, old_dir, new_dir)

    if not quiet:
        print('')
        print('Running command:')
        print('')
        print(cmd)
        print('')

    os.system(cmd)

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import hashlib


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    if i['input'].get('force_cache'):
        extra_cache_tags = i['input'].get('extra_cache_tags', '')
        r = automation.update_deps({
            'deps': meta['prehook_deps'] + meta['posthook_deps'],
            'update_deps': {
                'download-script': {
                    'extra_cache_tags': extra_cache_tags,
                    'force_cache': True
                },
                'extract-script': {
                    'extra_cache_tags': extra_cache_tags,
                    'force_cache': True
                }
            }
        })
        if r['return'] > 0:
            return r

    if env.get('CM_DOWNLOAD_LOCAL_FILE_PATH'):
        filepath = env['CM_DOWNLOAD_LOCAL_FILE_PATH']

        if not os.path.exists(filepath):
            return {'return': 1,
                    'error': 'Local file {} doesn\'t exist'.format(filepath)}

        env['CM_EXTRACT_REMOVE_EXTRACTED'] = 'no'

    if str(env.get('CM_DAE_EXTRACT_DOWNLOADED')
           ).lower() in ["yes", "1", "true"]:
        if (env.get('CM_EXTRACT_FINAL_ENV_NAME', '') == '') and (
                env.get('CM_DAE_FINAL_ENV_NAME', '') != ''):
            env['CM_EXTRACT_FINAL_ENV_NAME'] = env['CM_DAE_FINAL_ENV_NAME']

    return {'return': 0}


def postprocess(i):

    env = i['env']
    filepath = env.get('CM_EXTRACT_EXTRACTED_PATH', '')
    if filepath == '':
        filepath = env.get('CM_DOWNLOAD_DOWNLOADED_PATH', '')

    if filepath == '':
        return {'return': 1,
                'error': 'No extracted path set in "CM_EXTRACT_EXTRACTED_PATH"'}
    if not os.path.exists(filepath):
        return {'return': 1,
                'error': 'Extracted path doesn\'t exist: {}'.format(filepath)}

    if env.get('CM_DAE_FINAL_ENV_NAME'):
        env[env['CM_DAE_FINAL_ENV_NAME']] = filepath

    env['CM_GET_DEPENDENT_CACHED_PATH'] = filepath

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
from giturlparse import parse


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    meta = i['meta']
    automation = i['automation']

    repo = env.get('CM_MLPERF_RESULTS_GIT_REPO_URL', '')
    if repo.strip() == '':
        return {'return': 1, 'error': 'Invalid GIT_REPO_URL for MLPERF results'}

    branch = env.get('CM_GIT_BRANCH', '')
    if branch:
        extra_tags_string = f",_branch.{branch}"
    else:
        extra_tags_string = ""

    r = automation.update_deps({'deps': meta['prehook_deps'],
                                'update_deps': {
        'get-git-repo': {
            'tags': "_repo." + repo + extra_tags_string
        }
    }
    })
    if r['return'] > 0:
        return r
    env['CM_MLPERF_RESULTS_REPO_COMMIT_MESSAGE'] = env.get(
        'CM_MLPERF_RESULTS_REPO_COMMIT_MESSAGE', 'Added new results')

    p = parse(repo)
    if env.get('CM_GITHUB_PAT', '') != '':
        token = env['CM_GITHUB_PAT']
        env['CM_GIT_PUSH_CMD'] = f"""git push https://x-access-token:{env['CM_GITHUB_PAT']}@{p.host}/{p.owner}/{p.repo}"""
    else:
        env['CM_GIT_PUSH_CMD'] = "git push"

    return {'return': 0}


def postprocess(i):
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    if env.get('CM_DUMP_RAW_PIP_FREEZE_FILE_PATH', '') == '':
        env['CM_DUMP_RAW_PIP_FREEZE_FILE_PATH'] = os.path.join(
            os.getcwd(), "tmp-pip-freeze")

    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    os_info = i['os_info']

    automation = i['automation']

    pip_freeze = {}
    pip_freeze_file = env['CM_DUMP_RAW_PIP_FREEZE_FILE_PATH']
    if not os.path.isfile(pip_freeze_file):
        # If was not created, sometimes issues on Windows
        # There is another workaround
        if os_info['platform'] == 'windows':
            r = automation.cmind.access({'action': 'system',
                                         'automation': 'utils',
                                         'cmd': 'py -m pip freeze',
                                         'stdout': pip_freeze_file})
            # skip output

    if os.path.isfile(pip_freeze_file):
        with open(pip_freeze_file, "r") as f:
            for line in f.readlines():
                if "==" in line:
                    split = line.split("==")
                    pip_freeze[split[0]] = split[1].strip()

    state['pip_freeze'] = pip_freeze

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    env['CM_QAIC_COMPUTE_SDK_PATH'] = env['CM_GIT_CHECKOUT_PATH']

    '''
    if env.get('+PATH', []) == []:
        env['+PATH'] = []
    env['+PATH'].append(env['CM_LLVM_INSTALLED_PATH'])

    if env.get('+LD_LIBRARY_PATH', []) == []:
        env['+LD_LIBRARY_PATH'] = []
    env['+LD_LIBRARY_PATH'].append(os.path.join(env['CM_LLVM_INSTALLED_PATH'], "..", "lib"))
    '''
    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']
    # env['CM_QAIC_RUNNER_PATH'] = os.path.join(env['CM_QAIC_SOFTWARE_KIT_PATH'], "build", "utils", "qaic-runner")

    if '+PATH' not in env:
        env['+PATH'] = []

    env['CM_QAIC_COMPUTE_SDK_INSTALL_PATH'] = os.path.join(
        os.getcwd(),
        "src",
        "install",
        "qaic-compute-" +
        env['CM_QAIC_COMPUTE_SDK_INSTALL_MODE'])

    env['QAIC_COMPUTE_INSTALL_DIR'] = env['CM_QAIC_COMPUTE_SDK_INSTALL_PATH']

    env['+PATH'].append(os.path.join(env['CM_QAIC_COMPUTE_SDK_INSTALL_PATH'], "exec"))

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']
    meta = i['meta']

    if env.get('NVIDIA_ONLY', '') == 'yes':
        env['CM_GIT_URL'] = "https://github.com/GATEOverflow/nvidia-inference-code.git"

    if 'GITHUB_REPO_OWNER' in env and '<<<GITHUB_REPO_OWNER>>>' in env['CM_GIT_URL']:
        env['CM_GIT_URL'] = env['CM_GIT_URL'].replace(
            '<<<GITHUB_REPO_OWNER>>>', env['GITHUB_REPO_OWNER'])

    if 'CM_GIT_DEPTH' not in env:
        env['CM_GIT_DEPTH'] = ''

    if 'CM_GIT_RECURSE_SUBMODULES' not in env:
        env['CM_GIT_RECURSE_SUBMODULES'] = ''

    need_version = env.get('CM_VERSION', '')
    versions = meta['versions']

    if need_version != '' and not need_version in versions:
        env['CM_GIT_CHECKOUT'] = need_version

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    if env.get('CM_GIT_REPO_CURRENT_HASH', '') != '':
        env['CM_VERSION'] += "-git-" + env['CM_GIT_REPO_CURRENT_HASH']

#    env['CM_MLPERF_INFERENCE_RESULTS_PATH'] = os.path.join(os.getcwd(), "inference_results_"+env['CM_MLPERF_INFERENCE_RESULTS_VERSION_NAME'])

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import re


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    state = i['state']
    package_name = env['CM_APT_PACKAGE_NAME']

    install_cmd = env.get('CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD')
    if not install_cmd:
        return {
            'return': 1, 'error': 'Package manager installation command not detected for the given OS'}

    sudo = env.get('CM_SUDO', '')

    env['CM_APT_INSTALL_CMD'] = sudo + ' ' + install_cmd + ' ' + package_name

    if env.get('CM_APT_CHECK_CMD',
               '') != '' and env['CM_APT_INSTALL_CMD'] != '':
        env['CM_APT_INSTALL_CMD'] = f"""{env['CM_APT_CHECK_CMD']} || {env['CM_APT_INSTALL_CMD']}"""

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    need_version = env.get('CM_VERSION', '')
    if need_version == '':
        return {'return': 1,
                'error': 'internal problem - CM_VERSION is not defined in env'}

    print(recursion_spaces + '    # Requested version: {}'.format(need_version))

    version_split = need_version.split(".")
    while len(version_split) < 3:
        version_split.append("0")

    need_version = ".".join(version_split)

    host_os_bits = env['CM_HOST_OS_BITS']

    if os_info['platform'] != 'windows':
        host_os_machine = env['CM_HOST_OS_MACHINE']  # ABI

    # Prepare package name
    if os_info['platform'] == 'darwin':
        if host_os_bits != '64':
            return {'return': 1,
                    'error': 'this package doesn\'t support non 64-bit MacOS'}

        package_name = 'cmake-' + need_version + '-macos-universal.tar.gz'

    elif os_info['platform'] == 'windows':
        package_name = 'cmake-' + need_version + '-windows-'

        if host_os_bits == '64':
            package_name += 'x86_64'
        else:
            package_name += 'i386'

        package_name += '.zip'

    else:
        package_name = 'cmake-' + need_version + '-linux-'

        if host_os_machine.startswith(
                'arm') or host_os_machine.startswith('aarch'):
            if host_os_bits == '64':
                package_name += 'aarch64'
            else:
                return {'return': 1, 'error': 'this script doesn\'t support armv7'}
        else:
            package_name += 'x86_64'

        package_name += '.tar.gz'

    package_url = 'https://github.com/Kitware/CMake/releases/download/v' + \
        need_version + '/' + package_name

    print(recursion_spaces + '    # Prepared package URL: {}'.format(package_url))

    print('')
    print('Downloading from {} ...'.format(package_url))

    cm = automation.cmind

    r = cm.access({'action': 'download_file',
                   'automation': 'utils,dc2743f8450541e3',
                   'url': package_url})
    if r['return'] > 0:
        return r

    filename = r['filename']

    # Check what to do with this file depending on OS
    if os_info['platform'] == 'windows':
        print('Unzipping file {}'.format(filename))

        r = cm.access({'action': 'unzip_file',
                       'automation': 'utils,dc2743f8450541e3',
                       'strip_folders': 1,
                       'filename': filename})
        if r['return'] > 0:
            return r

        if os.path.isfile(filename):
            print('Removing file {}'.format(filename))
            os.remove(filename)

        path_bin = os.path.join(os.getcwd(), 'bin')
        path_include = os.path.join(os.getcwd(), 'include')
    elif os_info['platform'] == 'darwin':
        path_bin = os.path.join(os.getcwd(), 'CMake.app', 'Contents', 'bin')
        path_include = os.path.join(
            os.getcwd(), 'CMake.app', 'Contents', 'include')
    else:
        path_bin = os.path.join(os.getcwd(), 'bin')
        path_include = os.path.join(os.getcwd(), 'include')

    env['CM_CMAKE_PACKAGE'] = filename

    env['CM_CMAKE_INSTALLED_PATH'] = path_bin
    env['CM_GET_DEPENDENT_CACHED_PATH'] = os.getcwd()

    bin_name = 'cmake.exe' if os_info['platform'] == 'windows' else 'cmake'

    env['CM_CMAKE_BIN_WITH_PATH'] = os.path.join(path_bin, bin_name)

    # We don't need to check default paths here because we force install to
    # cache
    env['+PATH'] = [env['CM_CMAKE_INSTALLED_PATH']]

    if os.path.isdir(path_include):
        env['+C_INCLUDE_PATH'] = [path_include]

    return {'return': 0}



import os
import sys

if os.environ.get('CM_DATASET_REFERENCE_PREPROCESSOR', '1') == "0":
    import generic_preprocess
    generic_preprocess.preprocess()
else:
    mlperf_src_path = os.environ['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH']
    python_path = os.path.join(mlperf_src_path, "python")
    sys.path.insert(0, python_path)

    import imagenet
    import dataset

    dataset_path = os.environ['CM_DATASET_PATH']
    dataset_list = os.environ.get('CM_DATASET_IMAGES_LIST', None)
    img_format = os.environ.get('CM_DATASET_DATA_LAYOUT', 'NHWC')
    count = int(os.environ.get('CM_DATASET_SIZE', 1))
    preprocessed_dir = os.environ.get(
        'CM_DATASET_PREPROCESSED_PATH', os.getcwd())
    threads = os.environ.get('CM_NUM_THREADS', os.cpu_count())
    threads = int(os.environ.get('CM_NUM_PREPROCESS_THREADS', threads))

    if os.environ.get('CM_MODEL') == 'mobilenet':
        pre_process = dataset.pre_process_mobilenet
    elif os.environ.get('CM_MODEL', 'resnet50') == 'resnet50' and os.environ.get('CM_PREPROCESS_PYTORCH', '') == "yes":
        pre_process = dataset.pre_process_imagenet_pytorch
    elif os.environ.get('CM_MODEL', 'resnet50') == 'resnet50' and os.environ.get('CM_PREPROCESS_TFLITE_TPU', '') == "yes":
        pre_process = dataset.pre_process_imagenet_tflite_tpu
    else:
        pre_process = dataset.pre_process_vgg

    imagenet.Imagenet(data_path=dataset_path,
                      image_list=dataset_list,
                      name="imagenet",
                      image_format=img_format,
                      pre_process=pre_process,
                      use_cache=True,
                      count=count,
                      threads=threads,
                      preprocessed_dir=preprocessed_dir)



import shutil
import dataset
import openimages
import os
import sys
import os.path

mlperf_src_path = os.environ['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH']
python_path = os.path.join(mlperf_src_path, "python")
sys.path.insert(0, python_path)


dataset_path = os.environ['CM_DATASET_PATH']
preprocessed_dir = os.environ.get('CM_DATASET_PREPROCESSED_PATH', os.getcwd())

if os.environ.get('CM_DATASET_REFERENCE_PREPROCESSOR', '1') == "0":
    # import generic_preprocess
    # generic_preprocess.preprocess()
    import preprocess_object_detection_dataset as pp
    pp.preprocess()
else:
    dataset_list = os.environ.get('CM_DATASET_ANNOTATIONS_FILE_PATH', None)
    img_format = os.environ.get('CM_DATASET_DATA_LAYOUT', 'NHWC')
    count = int(os.environ.get('CM_DATASET_SIZE', 0)) or None
    image_width = int(os.environ.get('CM_DATASET_OPENIMAGES_RESIZE', 800))
    threads = os.environ.get('CM_NUM_THREADS', os.cpu_count())
    threads = os.environ.get('CM_NUM_PREPROCESS_THREADS', threads)
    name = "openimages-" + str(image_width) + "-retinanet"

    openimages.OpenImages(data_path=dataset_path,
                          image_list=dataset_list,
                          name=name,
                          image_format=img_format,
                          pre_process=dataset.pre_process_openimages_retinanet,
                          use_cache=True,
                          image_size=[image_width, image_width, 3],
                          count=count,
                          threads=threads,
                          preprocessed_dir=preprocessed_dir)

if os.environ["CM_DATASET_TYPE"] == "validation":
    src_path = os.environ.get(
        'CM_DATASET_ANNOTATIONS_DIR_PATH',
        os.path.join(
            dataset_path,
            "annotations"))
    dest_path = os.path.join(preprocessed_dir, "annotations")

    if not os.path.exists(dest_path):
        shutil.copytree(src_path, dest_path)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

#    print ('')
#    print ('Running preprocess function in customize.py ...')

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']


#    print ('')
#    print ('Running postprocess function in customize.py ...')

    # Saving predictions to JSON file to current directory
    # Should work with "cm docker script" ?

    data = state.get('cm_app_image_classification_onnx_py', {})

    fjson = 'cm-image-classification-onnx-py.json'
    fyaml = 'cm-image-classification-onnx-py.yaml'

    output = env.get('CM_APP_IMAGE_CLASSIFICATION_ONNX_PY_OUTPUT', '')
    if output != '':
        if not os.path.exists(output):
            os.makedirs(output)

        fjson = os.path.join(output, fjson)
        fyaml = os.path.join(output, fyaml)

    try:
        import json
        with open(fjson, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=4)
    except Exception as e:
        print('CM warning: {}'.format(e))

    try:
        import yaml
        with open(fyaml, 'w', encoding='utf-8') as f:
            yaml.dump(data, f)
    except Exception as e:
        print('CM warning: {}'.format(e))

    top_classification = data.get('top_classification', '')

    if env.get('CM_TMP_SILENT', '') != 'yes':
        if top_classification != '':
            print('')
            x = 'Top classification: {}'.format(top_classification)
            print('=' * len(x))
            print(x)

    return {'return': 0}



import cmind
import os
import sys

models = {
    "mobilenet": {
        "v1": {
            "multiplier": ["multiplier-1.0", "multiplier-0.75", "multiplier-0.5", "multiplier-0.25"],
            "resolution": ["resolution-224", "resolution-192", "resolution-160", "resolution-128"],
            "kind": [""]
        },
        "v2": {
            "multiplier": ["multiplier-1.0", "multiplier-0.75", "multiplier-0.5", "multiplier-0.35"],
            "resolution": ["resolution-224", "resolution-192", "resolution-160", "resolution-128"],
            "kind": [""]
        },
        "v3": {
            "multiplier": [""],
            "resolution": [""],
            "kind": ["large", "large-minimalistic", "small", "small-minimalistic"]
        }
    },
    "efficientnet": {
        "": {
            "multiplier": [""],
            "resolution": [""],
            "kind": ["lite0", "lite1", "lite2", "lite3", "lite4"]
        }
    }
}
variation_strings = {}
for t1 in models:
    variation_strings[t1] = []
    variation_list = []
    variation_list.append(t1)
    for version in models[t1]:
        variation_list = []
        if version.strip():
            variation_list.append("_" + version)
        variation_list_saved = variation_list.copy()
        for k1 in models[t1][version]["multiplier"]:
            variation_list = variation_list_saved.copy()
            if k1.strip():
                variation_list.append("_" + k1)
            variation_list_saved_2 = variation_list.copy()
            for k2 in models[t1][version]["resolution"]:
                variation_list = variation_list_saved_2.copy()
                if k2.strip():
                    variation_list.append("_" + k2)
                variation_list_saved_3 = variation_list.copy()
                for k3 in models[t1][version]["kind"]:
                    variation_list = variation_list_saved_3.copy()
                    if k3.strip():
                        variation_list.append("_" + k3)
                    variation_strings[t1].append(",".join(variation_list))
args = sys.argv

opt = None
if len(args) > 1:
    opt = args[1]
if opt == "submission":
    var = "_submission"
    execution_mode = "valid"
else:
    var = "_find-performance"
    execution_mode = "test"

precisions = ["fp32", "uint8"]
for model in variation_strings:
    for v in variation_strings[model]:
        for precision in precisions:
            if "small-minimalistic" in v and precision == "uint8":
                continue
            if model == "efficientnet" and precision == "uint8":
                precision = "int8"
            cm_input = {
                'action': 'run',
                'automation': 'script',
                'tags': f'generate-run-cmds,mlperf,inference,{var}',
                'quiet': True,
                'implementation': 'tflite-cpp',
                'precision': precision,
                'model': model,
                'scenario': 'SingleStream',
                'execution_mode': execution_mode,
                'test_query_count': '50',
                'adr': {
                        'tflite-model': {
                            'tags': v
                        },
                    'compiler': {
                            'tags': 'gcc'
                        },
                    'mlperf-inference-implementation': {
                            'tags': '_armnn,_use-neon'
                        }
                }
            }
            print(cm_input)
            r = cmind.access(cm_input)
            if r['return'] > 0:
                print(r)
                # exit(1)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    cm = automation.cmind

    path = os.getcwd()

    url = env['CM_PACKAGE_URL']
    env['CM_ML_MODEL_STARTING_WEIGHTS_FILENAME'] = url

    print('Downloading from {}'.format(url))

    r = cm.access({'action': 'download_file',
                   'automation': 'utils,dc2743f8450541e3',
                   'url': url})
    if r['return'] > 0:
        return r

    filename = r['filename']

    if env.get('CM_UNZIP') == "yes" or env.get('CM_UNTAR') == "yes":
        if env.get('CM_UNZIP') == "yes":
            cmd = "unzip "
        elif env.get('CM_UNTAR') == "yes":
            cmd = "tar -xvzf "
        os.system(cmd + filename)

        filename = env['CM_ML_MODEL_FILE']

        extract_folder = env.get('CM_EXTRACT_FOLDER', '')

        if extract_folder:
            env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(
                path, extract_folder, filename)
        else:
            env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(path, filename)
    else:
        env['CM_ML_MODEL_FILE'] = filename
        env['CM_ML_MODEL_FILE_WITH_PATH'] = r['path']

    env['CM_ML_MODEL_PATH'] = path

    if not os.path.exists(env['CM_ML_MODEL_FILE_WITH_PATH']):
        return {
            'return': 1, 'error': f"Model file path {env['CM_ML_MODEL_FILE_WITH_PATH']} not existing. Probably the model name {env['CM_ML_MODEL_FILE']} in model meta is wrong"}

    return {'return': 0}



import os
import json
from dmiparser import DmiParser

with open("meminfo.out", "r") as f:
    text = f.read()
    parser = DmiParser(text, sort_keys=True, indent=4)

    parsedStr = str(parser)
    parsedObj = json.loads(str(parser))
    memory = []

    ind = 0
    needed_global_keys = ['Speed', 'Configured Memory Speed', 'Type']
    added_global_keys = []
    needed_keys = ['Size', 'Rank']

    for item in parsedObj:
        if item['name'] == 'Physical Memory Array':
            ecc_value = item['props']['Error Correction Type']['values'][0]
            if not ecc_value or 'None' in ecc_value:
                ecc_value = "No ECC"
            memory.append({"info": ['Error Correction Type: ' + ecc_value]})
            ind += 1
            continue
        if item['name'] != 'Memory Device':
            continue
        memory.append({})
        memory[ind]['handle'] = item['handle']
        memory[ind]['info'] = []
        locator = item['props']['Locator']['values'][0]
        bank_locator = item['props']['Bank Locator']['values'][0]

        if not "Not Specified" in locator:
            memory[ind]['info'].append(locator)
        if not "Not Specified" in bank_locator:
            memory[ind]['info'].append(bank_locator)

        if item['props']['Size']['values'][0] == "No Module Installed":
            memory[ind]['populated'] = False
            memory[ind]['info'].append("Unpopulated")
        else:
            memory[ind]['populated'] = True

        for key in item['props']:
            if key in needed_global_keys and key not in added_global_keys:
                memory[0]['info'].append(
                    f'{key}: {";".join(item["props"][key]["values"])}')
                added_global_keys.append(key)
            elif key in needed_keys:
                memory[ind]['info'].append(
                    f'{key}: {";".join(item["props"][key]["values"])}')
        ind += 1

    meminfo = []
    for item in memory:
        meminfo.append("; ".join(item['info']))

    meminfo_string = ",   ".join(meminfo)
    with open("tmp-run-env.out", "w") as f:
        f.write(f"CM_HOST_MEM_INFO={meminfo_string}")



"""Convert weights from a gpt-j-6b model to a pax one.

Usage:

# Install the latest main branch of huggingface/transformers
pip3 install git+https://github.com/huggingface/transformers

# Get a checkpiont from the GPTJ family
https://huggingface.co/EleutherAI/gpt-j-6b

This points to
https://github.com/huggingface/transformers/blob/v4.30.2/src/transformers/models/gptj/modeling_flax_gptj.py
and in the default config, use_parallel_residual is true

# Example cmd:
python3 -m convert_gptj_ckpt --base EleutherAI/gpt-j-6b --pax pax_3b
"""
import argparse
import jax
from jax.experimental import pjit
import numpy as np
from paxml import checkpoints
from paxml import train_states
from praxis import py_utils
from transformers import AutoModelForCausalLM

# 6B example
num_layers = 28
num_heads = 16
dims_per_head = 256
vocab = 50401
num_gpus = 1


def convert(base_model_path, pax_model_path):
    """Convert from gpt-j-6b to pax."""
    print(f'Loading the base model from {base_model_path}')

    base = AutoModelForCausalLM.from_pretrained(
        base_model_path, low_cpu_mem_usage=True
    )
    for key, value in base.state_dict().items():
        print('%s %s' % (key, value.data.numpy().shape))

    jax_weights = {
        'lm': {
            'embedding_lookup': {
                'emb_var': base.state_dict()[
                    'transformer.wte.weight'
                ].data.numpy()[:vocab, :]
            },
            'softmax': {
                'logits_ffn': {
                    'linear': {
                        'w': (
                            base.state_dict()['lm_head.weight']
                            .data.numpy()
                            .transpose()[:, :vocab]
                        ),
                    },
                    'bias': {'b': base.state_dict()['lm_head.bias'].data.numpy()},
                }
            },
            'final_ln': {
                'scale': base.state_dict()[
                    'transformer.ln_f.weight'
                ].data.numpy(),
                'bias': base.state_dict()['transformer.ln_f.bias'].data.numpy(),
            },
            'transformer': {},
        }
    }

    for layer_idx in range(num_layers):
        query = base.state_dict()[
            'transformer.h.%d.attn.q_proj.weight' % layer_idx
        ].data.numpy()
        key = base.state_dict()[
            'transformer.h.%d.attn.k_proj.weight' % layer_idx
        ].data.numpy()
        value = base.state_dict()[
            'transformer.h.%d.attn.v_proj.weight' % layer_idx
        ].data.numpy()
        wc = np.stack((query, key, value))
        wc = np.reshape(
            wc, [3, num_heads, dims_per_head, num_heads * dims_per_head]
        )
        wc = np.transpose(wc, (0, 3, 1, 2))

        w_post = base.state_dict()[
            'transformer.h.%d.attn.out_proj.weight' % layer_idx
        ].data.numpy()
        w_post = np.reshape(
            w_post, [num_heads * dims_per_head, num_heads, dims_per_head]
        )
        layer_weight = {
            'self_attention': {
                'combined_qkv': {
                    'w': wc,
                },
                'post': {
                    'w': w_post,
                },
            },
            'ff_layer': {
                'ffn_layer1': {
                    'linear': {
                        'w': (
                            base.state_dict()[
                                'transformer.h.%d.mlp.fc_in.weight' % layer_idx
                            ]
                            .data.numpy()
                            .transpose()
                        ),
                    },
                    'bias': {
                        'b': base.state_dict()[
                            'transformer.h.%d.mlp.fc_in.bias' % layer_idx
                        ].data.numpy(),
                    },
                },
                'ffn_layer2': {
                    'linear': {
                        'w': (
                            base.state_dict()[
                                'transformer.h.%d.mlp.fc_out.weight' % layer_idx
                            ]
                            .data.numpy()
                            .transpose()
                        ),
                    },
                    'bias': {
                        'b': base.state_dict()[
                            'transformer.h.%d.mlp.fc_out.bias' % layer_idx
                        ].data.numpy(),
                    },
                },
            },
            'layer_norm': {
                'scale': base.state_dict()[
                    'transformer.h.%d.ln_1.weight' % layer_idx
                ].data.numpy(),
                'bias': base.state_dict()[
                    'transformer.h.%d.ln_1.bias' % layer_idx
                ].data.numpy(),
            },
        }
        jax_weights['lm']['transformer']['x_layers_%d' %
                                         layer_idx] = layer_weight

    print(f'Saving the pax model to {pax_model_path}')
    jax_states = train_states.TrainState(
        step=0, mdl_vars={'params': jax_weights}, opt_states={}
    )
    device_mesh = py_utils.create_device_mesh([1, 1, num_gpus])
    global_mesh = jax.sharding.Mesh(
        device_mesh, ['replica', 'data_mdl2', 'mdl'])

    # Identity pjit is needed to output a GDA model_states.
    def identity(x):
        return x

    pjitted_identity = pjit.pjit(
        identity,
        in_shardings=None,
        out_shardings=None)
    with global_mesh:
        jax_states_gda = pjitted_identity(jax_states)

    checkpoints.save_checkpoint(
        jax_states_gda,
        pax_model_path,
        checkpoint_type=checkpoints.CheckpointType.GDA,
    )
    print('done')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--base-model-path', type=str, required=True)
    parser.add_argument('--pax-model-path', type=str, required=True)
    args = parser.parse_args()

    convert(args.base_model_path, args.pax_model_path)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    automation = i['automation']
    run_script_input = i['run_script_input']

    recursion_spaces = i['recursion_spaces']

    conda_prefix_name = env.get('CM_CONDA_PREFIX_NAME', '')
    r = None
    file_name = 'conda.exe' if os_info['platform'] == 'windows' else 'conda'
    if conda_prefix_name == '':
        tmp_path = env.get('CM_CONDA_INSTALL_PATH', env.get('CM_TMP_PATH', ''))
        if tmp_path:
            x = ';' if os_info['platform'] == 'windows' else ':'
            tmp_path += x
        conda_path = os.path.join(os.path.expanduser("~"), "miniconda3", "bin")
        if os.path.exists(conda_path):
            tmp_path += os.path.join(os.path.expanduser("~"),
                                     "miniconda3", "bin")
        env['CM_TMP_PATH'] = tmp_path

        r = i['automation'].find_artifact({'file_name': file_name,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'PATH',
                                           'detect_version': True,
                                           'env_path_key': 'CM_CONDA_BIN_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
    else:
        env['CM_CONDA_INSTALL_PATH'] = os.path.join(os.getcwd(), "miniconda3")
        bin_dir = 'Scripts' if os_info['platform'] == 'windows' else 'bin'
        env['CM_CONDA_BIN_WITH_PATH'] = os.path.join(
            env['CM_CONDA_INSTALL_PATH'], bin_dir, file_name)

    if conda_prefix_name != '' or r['return'] > 0:
        if conda_prefix_name != '' or r['return'] == 16:
            if conda_prefix_name == '':
                if env.get('CM_TMP_FAIL_IF_NOT_FOUND', '').lower() == 'yes':
                    return r

                print(recursion_spaces + '    # {}'.format(r['error']))

            # Attempt to run installer
            r = automation.run_native_script(
                {'run_script_input': run_script_input, 'env': env, 'script_name': 'install'})
            if r['return'] > 0:
                return r

            # Grigori: temporal fix - should be generalized/improved above
            if os_info['platform'] == 'windows' and env.get(
                    'CM_CONDA_BIN_WITH_PATH', '') == '':
                env['CM_CONDA_INSTALL_PATH'] = os.path.join(
                    os.getcwd(), "miniconda3")
                env['CM_CONDA_BIN_WITH_PATH'] = os.path.join(
                    env['CM_CONDA_INSTALL_PATH'], 'Scripts', file_name)

    else:
        found_path = r['found_path']
        env['+PATH'] = [found_path]

    return {'return': 0}


def detect_version(i):
    r = i['automation'].parse_version({'match_text': r'conda\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_CONDA_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r
    return {'return': 0, 'version': r['version']}


def postprocess(i):
    env = i['env']

    r = detect_version(i)
    if r['return'] > 0:
        return r

    conda_bin_path = os.path.dirname(env['CM_CONDA_BIN_WITH_PATH'])
    env['CM_CONDA_BIN_PATH'] = conda_bin_path

    env['+PATH'] = [conda_bin_path]

    conda_prefix = os.path.dirname(conda_bin_path)
    env['CM_CONDA_PREFIX'] = conda_prefix
    env['CONDA_PREFIX'] = conda_prefix

    conda_lib_path = os.path.join(conda_prefix, "lib")

    if os.path.exists(conda_lib_path):
        env['CM_CONDA_LIB_PATH'] = conda_lib_path
        env['+LD_LIBRARY_PATH'] = [conda_lib_path]
        env['+LIBRARY_PATH'] = [conda_lib_path]

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}



import onnx
import os
import sys
import argparse
import yaml


def parse_args(add_help=True):
    parser = argparse.ArgumentParser(
        description='Print node precision info for the onnx file',
        add_help=add_help)
    parser.add_argument(
        '--input',
        default="retinanet.onnx",
        help='input onnx file')
    parser.add_argument(
        '--output',
        default="node-precision.yaml",
        help='output node precision file')
    args = parser.parse_args()

    return args


def main(args):

    onnx_model = onnx.load(args.input)
    list1 = [
        "/backbone/fpn/inner_blocks.0/Conv_output_0",
        "/head/classification_head/Sigmoid_output_0",
        "/head/classification_head/Sigmoid_1_output_0",
        "/head/classification_head/Sigmoid_2_output_0",
        "/head/classification_head/Sigmoid_3_output_0",
        "/head/classification_head/Sigmoid_4_output_0"
    ]
    list2 = [
        "1421",
        "1481",
        "1517",
        "1553",
        "1589",
        "1625",
    ]

    # check which list of node names is valid
    node_names = []
    valid_list = None

    # for n in enumerate_model_node_outputs(onnx_model):
    for n in onnx_model.graph.node:
        node_names.append(n.output[0])

    if set(list1) < set(node_names):
        valid_list = list1
    elif set(list2) < set(node_names):
        valid_list = list2
    else:
        print("Node names are not matching with the expected ones in the input onnx file.")
        sys.exit(1)

    node_precision_info = {}
    node_precision_info['FP16NodeInstanceNames'] = []

    fp16nodes = valid_list
    fp16nodes += ["boxes_1",
                  "boxes_2",
                  "boxes_3",
                  "boxes_4",
                  "boxes_5",
                  "scores_1",
                  "scores_2",
                  "scores_3",
                  "scores_4",
                  "scores_5"]

    # node_precision_info['FP16NodeInstanceNames'] = "["+", ".join(fp16nodes)+"]"
    node_precision_info['FP16NodeInstanceNames'] = fp16nodes

    yaml_output = yaml.safe_dump(node_precision_info, default_style=None)
    with open(args.output, "w") as f:
        f.write(yaml_output)

    print(f"Node precision info successfully printed out to {args.output}")


if __name__ == "__main__":
    args = parse_args()
    main(args)



import dataset
import criteo
import os
import sys
mlperf_dlrm_path = os.environ['CM_MLPERF_INFERENCE_DLRM_PATH']
python_path = os.path.join(mlperf_dlrm_path, "pytorch", "python")
sys.path.insert(0, python_path)


dataset_name = os.environ['CM_DATASET']
dataset_path = os.environ['CM_DATASET_PATH']
dataset_list = os.environ.get('CM_DATASET_IMAGES_LIST', None)
samples_to_aggregate_fix = os.environ.get(
    'CM_DATASET_SAMPLES_TO_AGGREGATE_FIX', None)
samples_to_aggregate_min = os.environ.get(
    'CM_DATASET_SAMPLES_TO_AGGREGATE_MIN', None)
samples_to_aggregate_max = os.environ.get(
    'CM_DATASET_SAMPLES_TO_AGGREGATE_MAX', None)
count = int(os.environ.get('CM_DATASET_SIZE', 0)) or None
max_ind_range = os.environ.get('CM_DATASET_MAX_IND_RANGE', -1)
threads = os.environ.get('CM_NUM_THREADS', os.cpu_count())
threads = os.environ.get('CM_NUM_PREPROCESS_THREADS', threads)

criteo.Criteo(data_path=dataset_path,
              name=dataset_name,
              pre_process=criteo.pre_process_criteo_dlrm,
              use_cache=True,
              samples_to_aggregate_fix=samples_to_aggregate_fix,
              samples_to_aggregate_min=samples_to_aggregate_min,
              samples_to_aggregate_max=samples_to_aggregate_max,
              max_ind_range=max_ind_range,
              count=count,
              mlperf_bin_loader=False,
              test_num_workers=threads
              )



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    datadir = env.get('CM_DATA_DIR', os.getcwd())
    env['CM_DATA_DIR'] = datadir

    env['MXNET_VER'] = env.get('CM_MXNET_VER', '22.08').replace("-", ".")

    env['CM_IMAGENET_LABELS_DOWNLOAD_DIR'] = env['CM_DATASET_IMAGENET_TRAIN_PATH']

    if env.get("CM_TMP_VARIATION", "") == "nvidia":
        code_path = os.path.join(
            env['CM_NVIDIA_DEEPLEARNING_EXAMPLES_REPO_PATH'],
            'MxNet',
            'Classification',
            'RN50v1.5')
        env['CM_RUN_DIR'] = code_path
        i['run_script_input']['script_name'] = "run-nvidia"

    elif env.get("CM_TMP_VARIATION", "") == "reference":
        code_path = os.path.join(
            env['CM_MLPERF_TRAINING_SOURCE'],
            'image_classification',
            'tensorflow2')
        env['CM_RUN_DIR'] = code_path
        i['run_script_input']['script_name'] = "run-reference"

    return {'return': 0}


def postprocess(i):

    env = i['env']

    data_dir = env['CM_DATA_DIR']
    env['CM_MLPERF_TRAINING_RESNET_DATA_PATH'] = data_dir

    env['CM_MLPERF_TRAINING_IMAGENET_PATH'] = env['CM_DATASET_IMAGENET_TRAIN_PATH']

    if env.get("CM_TMP_VARIATION", "") == "nvidia":
        env['CM_GET_DEPENDENT_CACHED_PATH'] = data_dir
        env['CM_MLPERF_TRAINING_NVIDIA_RESNET_PREPROCESSED_PATH'] = data_dir

    elif env.get("CM_TMP_VARIATION", "") == "reference":
        env['CM_GET_DEPENDENT_CACHED_PATH'] = os.path.join(
            data_dir, "tfrecords")
        env['CM_MLPERF_TRAINING_RESNET_TFRECORDS_PATH'] = os.path.join(
            data_dir, "tfrecords")

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] != 'windows':
        return {'return': 0}

    env = i['env']

    recursion_spaces = i['recursion_spaces']

    automation = i['automation']

    file_name = 'cl.exe'

    # Will check env['CM_TMP_PATH'] if comes from installation script
    ii = {'file_name': file_name,
          'env': env,
          'os_info': os_info,
          'default_path_env_key': 'PATH',
          'detect_version': True,
          'env_path_key': 'CM_CL_BIN_WITH_PATH',
          'run_script_input': i['run_script_input'],
          'recursion_spaces': recursion_spaces}

    rr = automation.find_artifact(ii)
    if rr['return'] > 0:
        # If not found in PATH, try a longer search
        if rr['return'] != 16:
            return rr

        if env.get('CM_INPUT', '').strip() == '' and env.get(
                'CM_TMP_PATH', '').strip() == '':

            print(
                i['recursion_spaces'] +
                '    Starting deep search for {} - it may take some time ...'.format(file_name))

            paths = ['C:\\Program Files\\Microsoft Visual Studio',
                     'C:\\Program Files (x86)\\Microsoft Visual Studio',
                     'C:\\Program Files (x86)\\Microsoft Visual Studio 14']

            restrict_paths = ['Hostx64\\x64']

            r = automation.find_file_deep({'paths': paths,
                                           'file_name': file_name,
                                           'restrict_paths': restrict_paths})
            if r['return'] > 0:
                return r

            found_paths = r['found_paths']

            if len(found_paths) == 0:
                return rr

            tmp_paths = ';'.join(found_paths)

            env['CM_TMP_PATH'] = tmp_paths
            env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

            ii['env'] = env

            rr = automation.find_artifact(ii)
            if rr['return'] > 0:
                return rr

        else:
            return rr

    found_path = rr['found_path']

    # Check vcvarall.bat
    state = i['state']
    script_prefix = state.get('script_prefix', [])

    # Attempt to find vcvars64.bat
    bat_file_name = 'VC\\Auxiliary\\Build\\vcvars64.bat'
    r = automation.find_file_back(
        {'path': found_path, 'file_name': bat_file_name})
    if r['return'] > 0:
        return r

    found_path_bat = r['found_path']

    if found_path_bat != '':
        path_to_vcvars = os.path.join(found_path_bat, bat_file_name)

        s = os_info['run_bat'].replace(
            '${bat_file}', '"' + path_to_vcvars + '"')

        script_prefix.append(s)

        state['script_prefix'] = script_prefix

    env['CM_CL_BIN'] = file_name
    env['CM_CL_BIN_WITH_PATH'] = os.path.join(found_path, file_name)

    # General compiler for general program compilation
    env['CM_C_COMPILER_BIN'] = file_name
    env['CM_C_COMPILER_WITH_PATH'] = os.path.join(found_path, file_name)
    env['CM_C_COMPILER_FLAG_OUTPUT'] = '/Fe:'
    env['CM_C_COMPILER_FLAG_VERSION'] = ''

    env['CM_CXX_COMPILER_BIN'] = env['CM_C_COMPILER_BIN']
    env['CM_CXX_COMPILER_WITH_PATH'] = env['CM_C_COMPILER_WITH_PATH']
    env['CM_CXX_COMPILER_FLAG_OUTPUT'] = '/Fe:'
    env['CM_CXX_COMPILER_FLAG_VERSION'] = ''

    return {'return': 0}


def detect_version(i):
    r = i['automation'].parse_version({'match_text': r'Version\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_CL_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']

    r = detect_version(i)

    if r['return'] > 0:
        return r

    version = r['version']

    env['CM_CL_CACHE_TAGS'] = 'version-' + version
    env['CM_COMPILER_CACHE_TAGS'] = 'version-' + version + ',family-msvc'
    env['CM_COMPILER_FAMILY'] = 'MSVC'
    env['CM_COMPILER_VERSION'] = env['CM_CL_VERSION']

    return {'return': 0, 'version': version}
